///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2006, 2007 Alberto González Palomo
//   Author: Alberto González Palomo <http://www.matracas.org>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////

function Kernel()
{
  var self = this;
  // Public functions:
  this.evaluate_step = evaluate_step;
  this.evaluate = evaluate;
  this.float_evaluate = float_evaluate;
  this.restart = restart;
  
  const openmath_namespace = "http://www.openmath.org/OpenMath";
  const    qmath_namespace = "http://www.matracas.org/ns/qmath";
  const   mathml_namespace = "http://www.w3.org/1998/Math/MathML";
  const    xhtml_namespace = "http://www.w3.org/1999/xhtml";
  
  const namespace_prefix_map =
    {
      "om"  : openmath_namespace,
      "q"   :    qmath_namespace,
      "m"   :   mathml_namespace,
      "h"   :    xhtml_namespace
    };
  
  const serializer = new XMLSerializer();
  
  var kernel_dir;
  var transformers;
  var alert_of_unhandled_cases;
  var abort_evaluation;
  
  // Constructor code:
  alert_of_unhandled_cases = false;
  abort_evaluation = false;
  transformers = {};
  kernel_dir = "chrome://algebra/content/kernel/";
  
  function evaluate_step(expression, destination_document, identifier)
  {
    try
      {
        var transformer;
        transformer = get_transformer(identifier);
        
        var result;
        if (destination_document)
          {
            result = transformer.transformToFragment(expression,
                                                     destination_document);
          }
        else
          {
            result = transformer.transformToDocument(expression);
          }
      }
    catch (exception)
      {
        var omstr;
        omstr = document.createElementNS(openmath_namespace, "OMSTR");
        omstr.appendChild(document.createTextNode(exception));
        result = document.createElementNS(openmath_namespace, "OMERR");
        result.appendChild(omstr);
      }
    
    return result;
  }
  
  function evaluate(expression, destination_document)
  {
    var result;
    var result_chain;// To show the steps.
    var result_string;
    result = expression;
    result_chain = new Array();
    result_string = serializer.serializeToString(result);
    var results;
    results = new Object();
    var step;
    step = 0;
    do
      {
        results[result_string] = step;
        result = evaluate_step(result, destination_document, "everything");
        result_chain.push(result);
        result_string = serializer.serializeToString(result);
        ++step;
      } while (! (result_string in results));
    
//     if (1 == expression.childNodes.length)
//       {
//         result = expression.firstChild;
//       }
//     else
//       {
//         // This should not happen normally, as the expression should be "clean",
//         // but it could be useful later, when we receive the expression from
//         // outside the program, e.g. pasted from the system clipboard.
//         result = expression.firstChild;
//         while (result && result.namespaceURI != openmath_namespace)
//           result = result.nextSibling;
//         if (!result) return expression;// Last recourse.
//       }
//    result = <om:OMOBJ><om:OMI>42</om:OMI></om:OMOBJ>
    
    return result;
  }
  
  function float_evaluate(expression, variable_map)
  {
    if (!expression) return NaN;
    if (abort_evaluation) return NaN;
    var result;
    if ("OMOBJ" == expression.localName) expression = expression.firstChild;
    try
      {
        var args;
        var args_length;
        var name;
        var i;
        switch (expression.localName)
          {
          case "OMA":
            args = expression.childNodes;
            args_length = args.length;
            name = (expression.firstChild.getAttribute("cd")
                    + ":" + expression.firstChild.getAttribute("name"));
            switch (name)
              {
              case "arith1:plus":
                result = float_evaluate(args[1], variable_map);
                for (i = 2; i < args_length; ++i)
                  result += float_evaluate(args[i], variable_map);
                break;
              case "arith1:minus":
                result = float_evaluate(args[1], variable_map);
                for (i = 2; i < args_length; ++i)
                  result -= float_evaluate(args[i], variable_map);
                break;
              case "arith1:unary_minus":
                result = -float_evaluate(args[1], variable_map);
                break;
              case "arith1:times":
                result = float_evaluate(args[1], variable_map);
                for (i = 2; i < args_length && result != 0.0; ++i)
                  result *= float_evaluate(args[i], variable_map);
                break;
              case "arith1:divide":
                result = float_evaluate(args[2], variable_map);
                for (i = 3; i < args_length && result != 0.0; ++i)
                  result *= float_evaluate(args[i], variable_map);
                if (result != 0.0)
                  result = float_evaluate(args[1], variable_map) / result;
                else
                  result = NaN;
                break;
              case "arith1:power":
                result = float_evaluate(args[1], variable_map);
                for (i = 2; i < args_length; ++i)
                  result = Math.pow(result,
                                    float_evaluate(args[i], variable_map));
                break;
              case "arith1:root":
                result = float_evaluate(args[1], variable_map);
                if (2 == Math.round(result))
                  {
                    result = Math.sqrt(float_evaluate(args[2], variable_map));
                  }
                else
                  {
                    result = Math.pow(float_evaluate(args[2], variable_map),
                                      1/result);
                  }
                break;
              case "arith1:abs":
                result = Math.abs(float_evaluate(args[1], variable_map));
                break;
              case "org.matracas.algebra:sqrt":
                result = Math.sqrt(float_evaluate(args[1], variable_map));
                break;
              case "transc1:exp":
                result = Math.exp(float_evaluate(args[1], variable_map));
                break;
              case "transc1:sin":
                result = Math.sin(float_evaluate(args[1], variable_map));
                break;
              case "transc1:cos":
                result = Math.cos(float_evaluate(args[1], variable_map));
                break;
              case "transc1:tan":
                result = Math.tan(float_evaluate(args[1], variable_map));
                break;
              case "transc1:arcsin":
                result = Math.asin(float_evaluate(args[1], variable_map));
                break;
              case "transc1:arccos":
                result = Math.acos(float_evaluate(args[1], variable_map));
                break;
              case "transc1:arctan":
                result = Math.atan(float_evaluate(args[1], variable_map));
                break;
              case "integer1:factorial":
                result = Math.round(float_evaluate(args[1], variable_map));
                for (i = result - 1; i > 1.5; --i) result *= i;
                break;
              default:
                if (alert_of_unhandled_cases) alert("name: " + name);
                result = NaN;
                break;
              }
            break;
          case "OMS":
            name = (expression.getAttribute("cd")
                    + ":" + expression.getAttribute("name"));
            switch (name)
              {
              case "nums1:pi": result = Math.PI; break;
              case "nums1:e":  result = Math.E;  break;
              case "nums1:pi": result = Math.PI; break;
              default:
                if (alert_of_unhandled_cases) alert("name: " + name);
                result = NaN;
                break;
              }
            break;
          case "OMV":
            name = expression.getAttribute("name");
            if (name in variable_map) result = variable_map[name];
            else                      result = NaN;
            break;
          case "OMI":
            result = parseInt(expression.textContent);
            break;
          case "OMF":
            // TODO: handle the hexadecimal notation for completeness.
            result = parseFloat(expression.getAttribute("dec"));
            break;
          default:
            if (alert_of_unhandled_cases)
              alert("localname: " + expression.localName);
            result = NaN;
            break;
          }
      }
    catch (exception)
      {
        //alert(exception);
        result = NaN;
      }
    
    return result;
  }
  
  function restart()
  {
    transformers = {};
  }
  
  function get_transformer(identifier)
  {
    var transformer;
    // Disallow files outside the directory.
    identifier = identifier.replace(/\.\.\//g, "");
    transformer = transformers[identifier];
    if (!transformer)
      {
        transformer = new XSLTProcessor();
        // It would be nice to be able to import stylesheets on demand, but
        // it seems that importing more than one stylesheet is not implemented:
        // Error: uncaught exception: [Exception... "Component returned failure code: 0x80004001 (NS_ERROR_NOT_IMPLEMENTED) [nsIXSLTProcessor.importStylesheet]"  nsresult: "0x80004001 (NS_ERROR_NOT_IMPLEMENTED)"  location: "JS frame :: chrome://algebra/content/Kernel.js :: init :: line 99"  data: no]
        var kernel_base_stylesheet;
        kernel_base_stylesheet = load_xml(kernel_dir + "base.xsl");
        
        var stylesheets_to_import;
        stylesheets_to_import = [identifier];
        for (var i in stylesheets_to_import)
          {
            var include;
            include = kernel_base_stylesheet
              .createElementNS("http://www.w3.org/1999/XSL/Transform",
                               "include");
            include.setAttribute("href", kernel_dir + stylesheets_to_import[i] + ".xsl");
            kernel_base_stylesheet.documentElement.appendChild(include);
            /*var stylesheet;
            stylesheet = load_xml(kernel_dir + stylesheets_to_import[i] + ".xsl");
            var template;
            template = stylesheet.documentElement.firstChild;
            while (template)
              {
                if ("template" == template.localName)
                  {
                    kernel_base_stylesheet.documentElement
                      .appendChild(kernel_base_stylesheet
                                   .importNode(template, true)
                                   );
                  }
                template = template.nextSibling;
              }
            */
          }
        
        transformer.importStylesheet(kernel_base_stylesheet);
        transformers[identifier] = transformer;
      }
    
    return transformer;
  }
  
  function load_xml(url)
  {
    var result;
    try
      {
        var request;
        request = new XMLHttpRequest();
        request.open("GET", url, false);
        request.send(null);
        result = request.responseXML;
      }
    catch (exception)
      {
        message("Error: " + url + ":\n" + exception);
      }
    
    return result;
  }
  
  function message(message)
  {
    alert(message);
  }
  
}
