///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2006 Alberto González Palomo
//   Author: Alberto González Palomo <http://www.matracas.org>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////

// Classes defined:
// - Plotter
// - Graphics_context_2d_SVG
// - Graphics_context_2d_EPS

function Plotter(kernel, plot_expressions, max_auto_width, max_auto_height)
{
  var self = this;
  // Public functions:
  this.get_panel = get_panel;
  this.slider_moved = slider_moved;// It was meant to be a slider.
  this.set_aspect = set_aspect;
  this.plot = plot;
  this.plot_in_context = plot_in_context;
  // Public properties:
  this.sample_count = 85;// Linked to internal_sample_count.
  this.cursor_index = 0;
  // They are public, but not abandoned:
  this.watch("sample_count", public_property_changed);
  this.watch("cursor_index", public_property_changed);
  
  const xul_namespace = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
  const openmath_namespace = "http://www.openmath.org/OpenMath";
  const   mathml_namespace = "http://www.w3.org/1998/Math/MathML";
  const    xhtml_namespace = "http://www.w3.org/1999/xhtml";
  
  const namespace_prefix_map =
    {
      "xul" :      xul_namespace,
      "om"  : openmath_namespace,
      "m"   :   mathml_namespace,
      "h"   :    xhtml_namespace
    };
  
  
  const aspect_threshold = 2.0;
  const palette = [[0, 0, 200],
                   [0, 180, 0],
                   [200, 0, 200],
                   [200, 180, 0],
                   [0, 180, 200]
                   ];
  
  // Surprisingly enough, it's faster to repain the plot in the canvas
  // than keeping it in an SVG behind it.
  const use_svg_plot_cache = false;
  
  var vertices;
  // It could be that the function does not have valid values in the whole
  // requested interval, so the graph could have to be split in segments.
  var segments;
  var u_interval_includes_first;
  var u_interval_includes_last;
  
  var overall_u_min;
  var overall_u_max;
  var overall_v_min;
  var overall_v_max;
  var current_aspect_name;
  
  var expressions;
  if ("Array" == plot_expressions.constructor.name)
    {
      expressions = plot_expressions;
    }
  else
    {
      expressions = new Array(plot_expressions);
    }
  
  var expression_count;
  compute_segments();
  
  var width;
  var height;
  var dimensions;
  dimensions = compute_canvas_dimensions("auto");
  width  = dimensions[0];
  height = dimensions[1];
  dimensions = null;
  
  var panel;
  var values_display;
  var aspect_choice;
  var canvas;
  var svg_plot;// For cacheing the static parts of the plot.
  var dom_parser;
  dom_parser = new DOMParser();
  
  panel = e(xul_namespace, "hbox",
            {"class":"plot-container"},
            [e(xul_namespace, "vbox",
               {"class":"plot-control-panel"},
               [aspect_choice =
                e(xul_namespace, "radiogroup",
                  {"oncommand":"this.plotter.set_aspect(this.value)"},
                  [e(xul_namespace, "radio",
                     {"label":"Autoscaled", "value":"auto", "selected":"true"}),
                   e(xul_namespace, "radio",
                     {"label":"Square coordinates", "value":"square"})
                   ]
                  ),
                build_sample_choices_menu(),
                values_display =
                e(xul_namespace, "vbox", {},
                  [/*
                   e(xul_namespace, "progressmeter",
                     {"value":"30",
                         "onmousemove":"slider_moved(this, event, 'cursor_index')"
                         }
                     ),
                   e(xul_namespace, "scrollbar",
                     {"curpos":"0", "maxpos":self.sample_count - 1,
                         "increment":"1",
                         // This is rather ugly, but "oncommand" only worked when
                         // clicking the arrows. It didn't work at all for the
                         // slider.
                         "oncommand":"slider_moved(this, event, 'cursor_index')",
                         "onmousedown":"slider_moved(this, event, 'cursor_index')",
                         "onmousemove":"slider_moved(this, event, 'cursor_index')",
                         "onmouseup":"slider_moved(this, event, 'cursor_index')",
                         "onclick":"slider_moved(this, event, 'cursor_index')"
                         }
                         )*/
                   ]
                  )
                ]
               ),
             e(xul_namespace, "vbox", {"class":"plot-view-panel"},
               [e(xul_namespace, "hbox",
                  {"class":"plot-label-container"},
                  [e(xul_namespace, "label",
                     {"flex":"1", "crop":"end", "class":"plot-label-center",
                         "value":"0"}
                     )
                   ]
                  ),
                e(xul_namespace, "box", { "class":"plot-area-container" },
                  [e(xul_namespace, "stack",
                     {"class":"plot-area", "context":"plot-area-context-menu", "onmousemove":"this.plotter.slider_moved(this, event, 'cursor_index')"},
                     [canvas = e(xhtml_namespace, "canvas",
                                 {"width":width, "height":height}
                                 )
                      ]
                     )
                   ]
                  ),
                e(xul_namespace, "hbox",
                  {"class":"plot-label-container"},
                  [e(xul_namespace, "label",
                     {"flex":"1", "crop":"end", "class":"plot-label-left",
                         "value":"0"}
                     ),
                   e(xul_namespace, "label",
                     {"flex":"1", "crop":"end", "class":"plot-label-center",
                         "value":"0"}
                     ),
                   e(xul_namespace, "label",
                     {"flex":"1", "crop":"end", "class":"plot-label-right",
                         "value":"0"}
                     )
                   ]
                  ),
                e(xul_namespace, "box", {"flex":"1"} /* Just a spacer. */)
                ]
               )
             ]
            );
  var i;
  for (i = 0; i < expression_count; ++i)
    {
      values_display.appendChild
        (e(xul_namespace, "hbox",
           {/*"style":"background:" + get_mixed_color(i, 255, 255, 255, 0.3)*/},
           [e(xul_namespace, "label",
              {"class":"value-display", "value":"x=",
                  "style":"color:" + get_color(i)}),
            e(xul_namespace, "textbox",
              {"class":"value-display", "size":"10", "readonly":"true"}),
            e(xul_namespace, "label",
              {"class":"value-display", "value":"y=",
                  "style":"color:" + get_color(i)}),
            e(xul_namespace, "textbox",
              {"class":"value-display", "size":"10", "readonly":"true"})
            ]
           )
         );
    }
  current_aspect_name = aspect_choice.value;
  
  set_plotter(panel);
  
  var repaint_needed;
  repaint_needed = true;
  
  // We have to delay this until Gecko has had some time to put the
  // elements in the screen. Otherwise, if we set the "value" property
  // of the textboxes for the coordinate values immediately, the values
  // never get displayed, even after setting the property again with
  // each repaint.
  setTimeout(plot, 200);
  
  
  function get_panel()
  {
    return panel;
  }
  
  function slider_moved(element, event, property)
  {
    var container;
    var value;
    var x;
    var y;
    switch (property)
      {
      case "cursor_index":
        switch (element.nodeName)
          {
          case "scrollbar":
            value = element.getAttribute('curpos');
            break;
          case "progressmeter":
            value = 100*(event.screenX - element.boxObject.screenX)/element.boxObject.width;
            element.value = value;
            value = value*self.sample_count/100;
            break;
          case "canvas":// canvas does not have boxObject.
            // This does not work right when the document is scrolled horizontally.
            //value = element.plotter.sample_count * (event.clientX - element.offsetLeft) / element.width;
            value = element.plotter.sample_count * (event.screenX - element.parentNode.boxObject.screenX) / element.width;
            break;
          case "stack":
            value = element.plotter.sample_count * (event.screenX - element.boxObject.screenX) / element.boxObject.width;
            break;
          }
        
        self.cursor_index = value;
        self.plot();
        break;
      }
  }
  
  function public_property_changed(property, old_value, new_value)
  {
    switch (property)
      {
      case "sample_count":
        if ("number" != typeof(new_value)) new_value = parseInt(new_value);
        if (new_value != old_value)
          {
            repaint_needed = true;
            if (new_value < 10) new_value = 10;
            invalidate_segments();
            // Keep the cursor around the same position:
            self.cursor_index = self.cursor_index * new_value / old_value;
          }
        break;
      case "cursor_index":
        new_value = Math.round(new_value);
        if (new_value < 0) new_value = 0;
        if (new_value >= self.sample_count) new_value = self.sample_count - 1;
        if (new_value != old_value) repaint_needed = true;
        break;
      default:
        if (new_value != old_value) repaint_needed = true;
        break;
      }
    
    return new_value;
  }
  
  function invalidate_segments()
  {
    segments = undefined;
    vertices = undefined;
    svg_plot = undefined;
  }
  
  function compute_segments()
  {
    segments = new Array();
    vertices = new Array();
    overall_u_min = Infinity;
    overall_u_max = -Infinity;
    overall_v_min = Infinity;
    overall_v_max = -Infinity;
    expression_count = 0;
    
    var exp;
    var expressions_with_same_u_range;
    var u_specification;
    var u;
    var u_name;
    var u_min;
    var u_max;
    for (var i = 0; i < expressions.length; ++i)
      {
        exp = xpath_first("om:*[2]", expressions[i]);
        u_specification = xpath_first("om:*[3]", expressions[i]);
        
        if (!u_specification || "OMA" != u_specification.localName)
          {
            error(ERROR_SYNTAX_U_RANGE);
            return;
          }
        else
          {
            switch (xpath_string("concat(om:*[1]/@cd, ' ', om:*[1]/@name)",
                                 u_specification))
              {
              case "list1 list":
              case "set1 set":
                u = xpath_first("om:*[2]", u_specification);
                if (u) u_name = u.getAttribute("name");
                u_min = kernel
                  .float_evaluate(xpath_first("om:*[3]", u_specification));
                u_max = kernel
                  .float_evaluate(xpath_first("om:*[4]", u_specification));
                u_interval_includes_first = true;
                u_interval_includes_last  = true;
                break;
              case "relation1 eq":
              case "set1 in":
                u = xpath_first("om:*[2]", u_specification);
                if (u) u_name = u.getAttribute("name");
                u_specification = xpath_first("om:*[3]", u_specification);
                u_min = kernel
                  .float_evaluate(xpath_first("om:*[2]", u_specification));
                u_max = kernel
                  .float_evaluate(xpath_first("om:*[3]", u_specification));
                switch (xpath_string("concat(om:*[1]/@cd, ' ', om:*[1]/@name)",
                                     u_specification))
                  {
                  case "integer1 integer_interval":
                  case "integer1 interval":
                  case "integer1 interval_cc":
                    u_interval_includes_first = true;
                    u_interval_includes_last  = true;
                    break;
                  case "integer1 interval_co":
                    u_interval_includes_first = true;
                    u_interval_includes_last  = false;
                    break;
                  case "integer1 interval_oc":
                    u_interval_includes_first = false;
                    u_interval_includes_last  = true;
                    break;
                  case "integer1 interval_oo":
                    u_interval_includes_first = false;
                    u_interval_includes_last  = false;
                    break;
                  default:
                    u_interval_includes_first = true;
                    u_interval_includes_last  = true;
                    break;
                  }
                break;
              default:
                error(ERROR_SYNTAX_U_RANGE);
                return;
              }
          }
        
        expressions_with_same_u_range = new Array();
        switch (xpath_string("concat(om:*[1]/@cd, ' ', om:*[1]/@name)", exp))
          {
          case "list1 list":
          case "set1 set":
            exp = xpath_first("om:*[2]", exp);
            while (exp)
              {
                expressions_with_same_u_range.push(exp);
                exp = exp.nextSibling;
              }
            break;
          default:
            expressions_with_same_u_range.push(exp);
            break;
          }
        
        for (var j in expressions_with_same_u_range)
          {
            add_segments_for_expression(expressions_with_same_u_range[j],
                                        u_name, u_min, u_max,
                                        get_alpha_color(expression_count, 0.5),
                                        get_color(expression_count));
            ++expression_count;
          }
      }
  }
  
  //function add_segments_for_expression(expression, fill_style, stroke_style)
  function add_segments_for_expression(exp, u_name, u_min, u_max, fill_style, stroke_style)
  {
    var vertex_index_base;
    vertex_index_base = vertices.length;
    var u_step;
    u_step = (u_max - u_min) * 1.0 / (self.sample_count - 1);
    var v_min;
    var v_max;
    v_min = Infinity;
    v_max = -Infinity;
    var variable_map;
    variable_map = {};
    var current_segment;
    current_segment = [];
    current_segment.fill_style   = fill_style;
    current_segment.stroke_style = stroke_style;
    segments.push(current_segment);
    var x;
    var y;
    var i_min;
    var i_max;
    if (u_interval_includes_first) i_min = 0;
    else                           i_min = 1;
    if (u_interval_includes_last) i_max = self.sample_count;
    else                          i_max = self.sample_count - 1;
    var i;
    for (i = 0; i < i_min; ++i) vertices.push([u_min + u_step * i, NaN]);
    for (i = i_min; i < i_max; ++i)
      {
        x = u_min + u_step * i;
        variable_map[u_name] = x;
        y = kernel.float_evaluate(exp, variable_map);
        vertices.push([x, y]);
        // We split the graph in any NaN or Infinity points. isFinite() tests
        // for both.
        if (!isFinite(x) || !isFinite(y))
          {
            if (current_segment.length > 0)
              {
                current_segment = [];
                current_segment.fill_style   = fill_style;
                current_segment.stroke_style = stroke_style;
                segments.push(current_segment);
              }
          }
        else
          {
            current_segment.push(vertex_index_base + i);
            if (y < v_min) v_min = y;
            if (y > v_max) v_max = y;
          }
      }
    while (i < self.sample_count) vertices.push([u_min + u_step * (i++), NaN]);
    if (0 == current_segment.length) segments.pop();
    // The algorithm above ensures that there is no void segment, which
    // is required for plot().
    if (u_min < overall_u_min) overall_u_min = u_min;
    if (u_max > overall_u_max) overall_u_max = u_max;
    if (v_min < overall_v_min) overall_v_min = v_min;
    if (v_max > overall_v_max) overall_v_max = v_max;
  }
  
  function plot()
  {
    if (!repaint_needed) return;
    repaint_needed = false;
    
    var c;
    
    if (use_svg_plot_cache && !svg_plot)
      {
        c = new Graphics_context_2d_SVG();
        plot_in_context(c, canvas.width, -canvas.height);
        var plot_area;
        plot_area = canvas.parentNode;
        while (canvas.previousSibling)
          {
            plot_area.removeChild(canvas.previousSibling);
          }
        svg_plot = dom_parser.parseFromString(c.toString(), "text/xml");
        plot_area.insertBefore(svg_plot.documentElement, canvas);
      }
    
    c = canvas.getContext("2d");
    c.clearRect(0, 0, canvas.width, canvas.height);
    if (! svg_plot) plot_in_context(c, canvas.width, -canvas.height);
    plot_cursors(c, canvas.width, -canvas.height);
    var label_container;
    var label;
    
    label_container = canvas.parentNode.parentNode.nextSibling;
    label = label_container.firstChild;
    label.setAttribute("value", overall_u_min.toFixed(4));
    label = label.nextSibling;
    label.setAttribute("value", overall_v_min.toFixed(4));
    label = label.nextSibling;
    label.setAttribute("value", overall_u_max.toFixed(4));
    
    label_container = canvas.parentNode.parentNode.previousSibling;
    label = label_container.firstChild;
    label.setAttribute("value", overall_v_max.toFixed(4));
  }
  
  function plot_in_context(c, width, height)
  {
    if (!width || !height) return;
    
    if (!segments) compute_segments();
    
    var u_range;
    var v_range;
    u_range = overall_u_max - overall_u_min;
    v_range = overall_v_max - overall_v_min;
    // The ranges can be 0 if the function is computed in one point only.
    if (u_range < 0.0000001) u_range = 1.0;
    if (v_range < 0.0000001) v_range = 1.0;
    
    var scale_x;
    scale_x = width / u_range;
    var scale_y;
    // In the canvas and SVG context, we need to invert the y axis, but not
    // for Postscript. Since the Postscript case is nicer, I don't want to
    // put there a double inversion, so the coordinate flip for canvas and
    // SVG should be done by giving a negative height.
    //scale_y = -height / v_range;// Invert the axis.
    scale_y = height / v_range;
    
    width  = Math.abs(width);
    height = Math.abs(height);
    
    var x;
    var y;
    
    c.save();
    c.translate(width  / 2 - scale_x*(overall_u_min + u_range/2),
                height / 2 - scale_y*(overall_v_min + v_range/2));
    // Using the transformation matrix for scaling also affects the line
    // width, which we want to avoid in this case, so we scale the
    // coordinates ourselves.
    // It is possible to avoid the like width problem by saving the context
    // after building the path but before stroking it, but then we still
    // have to correct the circle cursor. For now this seems simpler to me.
    //c.scale(scale_x, -scale_y);
    var vertex;
    for (var i = 0; i < vertices.length; ++i)
      {
        vertex = vertices[i];
        vertex[2] = scale_x*vertex[0];
        vertex[3] = scale_y*vertex[1];
      }
    
    // Background grid:
    c.strokeStyle = "rgb(200, 200, 200)";
    c.beginPath();
    var x_min;
    var x_max;
    var y_min;
    var y_max;
    x_min = scale_x*overall_u_min;
    x_max = scale_x*overall_u_max;
    y_min = scale_y*overall_v_min;
    y_max = scale_y*overall_v_max;
    var i;
    var grid_step;
    i = Math.floor(overall_v_max) - Math.ceil(overall_v_min);
    grid_step = scale_y;
    while (i > height)
      {
        i = i / 10.0;
        grid_step *= 10.0;
      }
    y = (scale_y * Math.ceil(overall_v_min));
    y = Math.ceil(y / grid_step) * grid_step;
    while (i >= 0)
      {
        c.moveTo(x_min, y);
        c.lineTo(x_max, y);
        y += grid_step;
        --i;
      }
    i = Math.floor(overall_u_max) - Math.ceil(overall_u_min);
    grid_step = scale_x;
    while (i > height)
      {
        i = i / 10.0;
        grid_step *= 10.0;
      }
    x = (scale_x * Math.ceil(overall_u_min));
    x = Math.ceil(x / grid_step) * grid_step;
    while (i >= 0)
      {
        c.moveTo(x, y_min);
        c.lineTo(x, y_max);
        x += grid_step;
        --i;
      }
    c.stroke();
    // Axis:
    c.strokeStyle = "rgb(10, 10, 10)";
    c.beginPath();
    c.moveTo(scale_x*overall_u_min, 0);
    c.lineTo(scale_x*overall_u_max, 0);
    c.moveTo(0, scale_y*overall_v_min);
    c.lineTo(0, scale_y*overall_v_max);
    c.stroke();
    
    // Draw graph, one segment at a time:
    c.lineWidth = 2.0;
    var current_segment;
    var vertex;
    for (var s = 0; s < segments.length; ++s)
      {
        current_segment = segments[s];
        c.fillStyle   = current_segment.fill_style;
        c.strokeStyle = current_segment.stroke_style;
        
        // Fill:
        c.beginPath();
        vertex = vertices[current_segment[0]];
        c.moveTo(vertex[2], vertex[3]);
        for (i = 1; i < current_segment.length; ++i)
          {
            vertex = vertices[current_segment[i]];
            c.lineTo(vertex[2], vertex[3]);
          }
        c.lineTo(vertex[2], 0);
        c.lineTo(vertices[current_segment[0]][2], 0);
        c.fill();
        
        // Stroke:
        c.beginPath();
        vertex = vertices[current_segment[0]];
        c.moveTo(vertex[2], vertex[3]);
        for (i = 1; i < current_segment.length; ++i)
          {
            vertex = vertices[current_segment[i]];
            c.lineTo(vertex[2], vertex[3]);
          }
        c.stroke();
      }
    
    // Blue pill:
    c.restore();
  }
  
  function plot_cursors(c, width, height)
  {
    var u_range;
    var v_range;
    u_range = overall_u_max - overall_u_min;
    v_range = overall_v_max - overall_v_min;
    // The ranges can be 0 if the function is computed in one point only.
    if (u_range < 0.0000001) u_range = 1.0;
    if (v_range < 0.0000001) v_range = 1.0;
    
    var scale_x;
    scale_x = width / u_range;
    var scale_y;
    scale_y = height / v_range;
    
    width  = Math.abs(width);
    height = Math.abs(height);
    
    c.save();
    c.translate(width  / 2 - scale_x*(overall_u_min + u_range/2),
                height / 2 - scale_y*(overall_v_min + v_range/2));
    
    var vertex;
    var i;
    var x;
    var y;
    
    c.fillStyle = "rgba(200, 0, 0, 0.4)";
    var cursor_position_display;
    cursor_position_display = values_display.firstChild;
    for (i = self.cursor_index;
         i < vertices.length;
         i += self.sample_count)
      {
        vertex = vertices[i];
        //x = vertices[self.cursor_index][0];
        //y = vertices[self.cursor_index][1];
        x = vertex[2];
        y = vertex[3];
        if (isFinite(x) && isFinite(y))
          {
            //c.moveTo(scale_x*x, scale_y*y);
            //c.arc(scale_x*x, scale_y*y, 10, 0, 2*Math.PI, true);
            c.beginPath();
            c.moveTo(x, y);
            c.arc(x, y, 10, 0, 2*Math.PI, true);
            c.fill();
          }
        if (cursor_position_display)
          {
            x = vertex[0].toFixed(8).replace("Infinity", "\u221E");
            y = vertex[1].toFixed(8).replace("Infinity", "\u221E");
            cursor_position_display.childNodes[1].value = x;
            cursor_position_display.childNodes[3].value = y;
            cursor_position_display = cursor_position_display.nextSibling;
          }
      }
    
    c.restore();
  }
  
  function get_color(index)
  {
    var c;
    c = palette[index % palette.length];
    return "rgb(" + c[0] + "," + c[1] + "," + c[2] + ")";
  }
  
  function get_alpha_color(index, alpha)
  {
    var c;
    c = palette[index % palette.length];
    return "rgba(" + c[0] + "," + c[1] + "," + c[2] + "," + alpha + ")";
  }
  
  function get_mixed_color(index, r, g, b, alpha)
  {
    var c;
    c = palette[index % palette.length];
    r = Math.round(c[0]*alpha + r*(1.0-alpha)) % 256;
    g = Math.round(c[1]*alpha + g*(1.0-alpha)) % 256;
    b = Math.round(c[2]*alpha + b*(1.0-alpha)) % 256;
    return "rgb(" + r + "," + g + "," + b + ")";
  }
  
  function set_plotter(node)
  {
    node.plotter = self;
    node = node.firstChild;
    while (node) { set_plotter(node); node = node.nextSibling; }
  }
  
  function compute_canvas_dimensions(aspect_name)
  {
    current_aspect_name = aspect_name;
    
    if (!segments) compute_segments();
    
    var u_range;
    var v_range;
    u_range = overall_u_max - overall_u_min;
    v_range = overall_v_max - overall_v_min;
    // The ranges can be 0 if the function is computed in one point only.
    if (u_range < 0.0000001) u_range = 1.0;
    if (v_range < 0.0000001) v_range = 1.0;
    
    var width;
    var height;
    width = max_auto_width;
    if ("square" == aspect_name)
      {
        height = v_range * width / u_range;
      }
    else // "auto"
      {
        if (u_range/v_range > aspect_threshold)
          {
            height = width / aspect_threshold;
          }
        else if (v_range/u_range > aspect_threshold)
          {
            height = width * aspect_threshold;
          }
        else
          {
            height = v_range * width / u_range;
          }
      }
    
    if (height > max_auto_height)
      {
        width /= height / max_auto_height;
        height = max_auto_height;
      }
    
    return [width, height];
  }
  
  function set_aspect(aspect_name)
  {
    var dimensions;
    dimensions = compute_canvas_dimensions(aspect_name);
    
    if (!dimensions[0] || !dimensions[1]) return;
    
    //var new_canvas;
    //new_canvas = e(xhtml_namespace, "canvas",
    //               {"width":dimensions[0], "height":dimensions[1]}
    //               );
    //canvas.parentNode.replaceChild(new_canvas, canvas);
    //canvas = new_canvas;
    canvas.width  = dimensions[0];
    canvas.height = dimensions[1];
    repaint_needed = true;
    plot();
  }
  
  function build_sample_choices_menu()
  {
    var sample_count_menu;
    sample_count_menu =
      e(xul_namespace, "menulist",
        {"oncommand":"this.plotter.sample_count = this.value; this.previousSibling.doCommand()", "label":"Grid"}
        );
    var sample_choices;
    sample_choices = [15, 30, 45, 60, 85,
                      100, 125, 150, 196,
                      200, 240, 275,
                      300];
    var sample_choices_containter;
    sample_choices_containter = e(xul_namespace, "menupopup");
    for (var i in sample_choices)
      {
        var value;
        value = String(sample_choices[i]);
        sample_choices_containter
          .appendChild(e(xul_namespace, "menuitem",
                         {"label":value + " samples", "value":value}
                         ));
      }
    sample_count_menu.appendChild(sample_choices_containter);
    sample_count_menu.setAttribute("value", String(self.sample_count));
    
    return sample_count_menu;
  }
  
  const ERROR_SYNTAX_U_RANGE = 1;
  function error(code)
  {
    // TODO: get a translated error message.
    switch (code)
      {
      case ERROR_SYNTAX_U_RANGE:
        alert("Plot: the range for the variable was not specified correctly.");
        break;
      }
  }
  
}

function Graphics_context_2d_SVG()
{
  var self = this;
  
  // Public functions:
  this.save = save;
  this.restore = restore;
  this.clearRect = clearRect;
  this.translate = translate;
  this.scale = scale;
  this.beginPath = beginPath;
  this.moveTo = moveTo;
  this.lineTo = lineTo;
  this.arc = arc;
  this.closePath = closePath;
  this.stroke = stroke;
  this.fill = fill;
  this.toString = toString;
  // Public properties:
  this.lineWidth = 1.0;
  this.strokeStyle = undefined;
  this.fillStyle   = undefined;
  this.watch("lineWidth",   public_property_changed);
  this.watch("strokeStyle", public_property_changed);
  this.watch("fillStyle",   public_property_changed);
  
  var content;
  content = "";
  var x_min;
  var x_max;
  var y_min;
  var y_max;
  x_min = Infinity;
  x_max = -Infinity;
  y_min = Infinity;
  y_max = -Infinity;
  var path;
  path = "";
  var context_stack;
  context_stack = new Array();
  function context(model)
  {
    if (model)
      {
        this.transformation = "";// Special case: difference to previous matrix.
        this.line_width     = model.line_width;
        this.stroke_style   = model.stroke_style;
        this.fill_style     = model.fill_style;
      }
    else
      {
        this.transformation = undefined;
        this.line_width = 1.0;
        this.stroke_style = undefined;
        this.fill_style   = undefined;
      }
  }
  context_stack.push(new context());
  var current_context;
  current_context = context_stack[context_stack.length - 1];
  
  function public_property_changed(property, old_value, new_value)
  {
    switch (property)
      {
      case "lineWidth":
        if (current_context) current_context.line_width = new_value;
        break;
      case "strokeStyle":
        if (current_context)
          {
            current_context.stroke_style = translate_style(new_value);
          }
        break;
      case "fillStyle":
        if (current_context)
          {
            current_context.fill_style = translate_style(new_value);
          }
        break;
      default:
        // Ignore any unsupported properties.
        break;
      }
  }
  
  function save()
  {
    context_stack.push(new context(current_context));
    current_context = context_stack[context_stack.length - 1];
  }
  function restore()
  {
    if (current_context.transformation)
      {
        content = "<g transform=\"" + current_context.transformation + "\">\n"
          + content
          + "</g>\n";
      }
    context_stack.pop();
    current_context = context_stack[context_stack.length - 1];
  }
  function clearRect(x, y, w, h)
  {
    // I don't know what to do here. Painting white does not seem so
    // appropriate. Since this function is used in the plotter just for
    // clearing the canvas before painting the current frame, we can
    // ignore it for now.
  }
  function translate(x, y)
  {
    if (current_context.transformation)
      {
        current_context.transformation +=
          " translate(" + format(x) + ", " + format(y) + ")";
      }
    else
      {
        current_context.transformation =
          " translate(" + format(x) + ", " + format(y) + ")";
      }
  }
  function scale(x, y)
  {
    if (current_context.transformation)
      {
        current_context.transformation +=
          " scale(" + format(x) + ", " + format(y) + ")";
      }
    else
      {
        current_context.transformation =
          " scale(" + format(x) + ", " + format(y) + ")";
      }
  }
  function beginPath() { path = ""; }
  function moveTo(x, y) {
    if (path) path += " M" + format(x) + "," + format(y);
    else      path =  "M"  + format(x) + "," + format(y);
    if (x < x_min) x_min = x;
    if (x > x_max) x_max = x;
    if (y < y_min) y_min = y;
    if (y > y_max) y_max = y;
  }
  function lineTo(x, y) {
    if (path) path += " L" + format(x) + "," + format(y);
    else      path =  "L"  + format(x) + "," + format(y);
    if (x < x_min) x_min = x;
    if (x > x_max) x_max = x;
    if (y < y_min) y_min = y;
    if (y > y_max) y_max = y;
  }
  function arc(x, y, radius, begin, end, clockwise)
  {
    // Leave this for now. We have to compute the coordinates of the
    // start and end points of the arc.
    //path += " A" + radius + "," + radius + " 0 1 0 " + begin + "," + end + " " + x + "," + y;
    if (x - Math.abs(radius) < x_min) x_min = x - Math.abs(radius);
    if (x + Math.abs(radius) > x_max) x_max = x + Math.abs(radius);
    if (y - Math.abs(radius) < y_min) y_min = y - Math.abs(radius);
    if (y + Math.abs(radius) > y_max) y_max = y + Math.abs(radius);
  }
  function closePath() { path += " Z"; }
  function stroke()
  {
    if (current_context.stroke_style && path)
      {
        content += "<path d=\"" + path + "\""
          + " style=\""
          + "stroke:" + current_context.stroke_style + ";"
          + "stroke-width:" + current_context.line_width + ";"
          + "fill:none;"
          + "\""
          + "/>\n";
      }
  }
  function fill()
  {
    if (current_context.fill_style && path)
      {
        content += "<path d=\"" + path + "\""
          + " style=\""
          + "fill:" + current_context.fill_style + ";"
          + "\""
          + "/>\n";
      }
  }
  
  function format(n)
  {
    return n.toFixed(5);
  }
  function translate_style(style)
  {
    var match;
    match = style.match(/^rgb\(([^,]+),([^,]+),([^,\)]+)\)/);
    if (match)
      {
        return ("rgb(" + match[1] + ", " + match[2] + ", " + match[3] + ")"
                ).replace(/\s+/g, " ");
      }
    
    match = style.match(/^rgba\(([^,]+),([^,]+),([^,]+),([^,\)]+)\)/);
    if (match)
      {
        return ("rgb(" + match[1] + ", " + match[2] + ", " + match[3] + ")"
                + ";opacity:" + match[4]
                ).replace(/\s+/g, " ");
      }
    
    return undefined;
  }
  
  function toString()
  {
    var width;
    var height;
    width  = x_max - x_min;
    height = y_max - y_min;
    return ("<?xml version=\"1.0\" standalone=\"yes\"?>\n<!-- Created with Algebra (http://www.matracas.org) -->\n<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n<svg width=\"" + width + "px\" height=\"" + height + "px\" viewBox=\"0 0 " + width + " " + height + "\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">\n"
            + content
            + "</svg>\n"
            );
  }
  
}

function Graphics_context_2d_EPS()
{
  var self = this;
  
  // Public functions:
  this.save = save;
  this.restore = restore;
  this.clearRect = clearRect;
  this.translate = translate;
  this.scale = scale;
  this.beginPath = beginPath;
  this.moveTo = moveTo;
  this.lineTo = lineTo;
  this.arc = arc;
  this.closePath = closePath;
  this.stroke = stroke;
  this.fill = fill;
  this.toString = toString;
  // Public properties:
  this.lineWidth = 1.0;
  this.strokeStyle = undefined;
  this.fillStyle   = undefined;
  this.watch("lineWidth",   public_property_changed);
  this.watch("strokeStyle", public_property_changed);
  this.watch("fillStyle",   public_property_changed);
  
  var content;
  content = "";
  var x_min;
  var x_max;
  var y_min;
  var y_max;
  x_min = Infinity;
  x_max = -Infinity;
  y_min = Infinity;
  y_max = -Infinity;
  var stroke_style;
  var fill_style;
  
  function public_property_changed(property, old_value, new_value)
  {
    switch (property)
      {
      case "lineWidth":
        content += String(new_value) + " setlinewidth\n";
        break;
      case "strokeStyle":
        stroke_style = translate_style(new_value);
        break;
      case "fillStyle":
        fill_style = translate_style(new_value);
        break;
      default:
        // Ignore any unsupported properties.
        break;
      }
    
    return new_value;
  }
  
  function save()
  { content += "gsave\n"; }
  function restore()
  { content += "grestore\n"; }
  function clearRect(x, y, w, h)
  {
    // I don't know what to do here. Painting white does not seem so
    // appropriate. Since this function is used in the plotter just for
    // clearing the canvas before painting the current frame, we can
    // ignore it for now.
  }
  function translate(x, y)
  { content += format(x) + " " + format(y) + " translate\n"; }
  function scale(x, y)
  { content += format(x) + " " + format(y) + " scale\n"; }
  function beginPath() { content += "newpath\n" }
  function moveTo(x, y)
  {
    content += format(x) + " " + format(y) + " moveto\n";
    if (x < x_min) x_min = x;
    if (x > x_max) x_max = x;
    if (y < y_min) y_min = y;
    if (y > y_max) y_max = y;
  }
  function lineTo(x, y)
  {
    content += format(x) + " " + format(y) + " lineto\n";
    if (x < x_min) x_min = x;
    if (x > x_max) x_max = x;
    if (y < y_min) y_min = y;
    if (y > y_max) y_max = y;
  }
  function arc(x, y, radius, begin, end, clockwise)
  {
    content +=
      format(x) + " " + format(y) + " " + format(radius) + " " +
      begin + " " + end;// begin and end need as much precision as possible.
    if (clockwise) content += " arcn\n";
    else           content += " arc\n";
    if (x - Math.abs(radius) < x_min) x_min = x - Math.abs(radius);
    if (x + Math.abs(radius) > x_max) x_max = x + Math.abs(radius);
    if (y - Math.abs(radius) < y_min) y_min = y - Math.abs(radius);
    if (y + Math.abs(radius) > y_max) y_max = y + Math.abs(radius);
  }
  function closePath() { content += "closepath\n" }
  function stroke()
  {
    if (stroke_style) content += stroke_style + "\nstroke\n";
  }
  function fill()
  {
    if (fill_style) content += fill_style + "\neofill\n";
  }
  
  function format(n)
  {
    return n.toFixed(5);
  }
  function translate_style(style)
  {
    var match;
    match = style.match(/^rgb\(([^,]+),([^,]+),([^,\)]+)\)/);
    if (match)
      {
        return (format(match[1]/255) + " " +
                format(match[2]/255) + " " +
                format(match[3]/255) +
                " setrgbcolor"
                ).replace(/\s+/g, " ");
      }
    
    match = style.match(/^rgba\(([^,]+),([^,]+),([^,]+),([^,\)]+)\)/);
    if (match)
      {
        // Postscript does not have transparency.
        if (parseFloat(match[4]) > 0.5)
          {
            return (match[1]/255 + " " + match[2]/255 + " " + match[3]/255 +
                    " setrgbcolor"
                    ).replace(/\s+/g, " ");
          }
      }
    
    return undefined;
  }
  
  function toString()
  {
    var width;
    var height;
    width  = x_max - x_min;
    height = y_max - y_min;
    return ("%!PS-Adobe-3.0 EPSF-3.0\n%%Creator: Algebra 1.0 http://www.matracas.org/algebra\n%%BoundingBox: 0 0 " + Math.ceil(width) + " " + Math.ceil(height) + "\n%%HiResBoundingBox: 0.0000000 0.0000000 " + width.toFixed(7) + " " + height.toFixed(7) + "\n%%EndComments\n"
            + content
            + "\nshowpage\n%EOF\n"
            );
  }
  
}
