///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2005 Alberto González Palomo
//   Author: Alberto González Palomo <http://www.matracas.org>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////

// jsxls.js - convenience functions to build XSL templates from JavaScript.

const xsl_namespace = "http://www.w3.org/1999/XSL/Transform";

const XSL =
  {
    template: function (pattern, name, content)
    {
      var node;
      node = document.createElementNS(xsl_namespace, "template");
      if (pattern) node.setAttribute("match", pattern);
      if (name)    node.setAttribute("name", name);
      this.add_content(node, content);
      return node;
    },
    
    call_template: function (name, content)
    {
      var node;
      node = document.createElementNS(xsl_namespace, "call-template");
      if (name)    node.setAttribute("name", name);
      this.add_content(node, content);
      return node;
    },
    
    apply_templates: function (select, content)
    {
      var node;
      node = document.createElementNS(xsl_namespace, "apply-templates");
      if (select)    node.setAttribute("select", select);
      this.add_content(node, content);
      return node;
    },
    
    param: function (name, content)
    {
      var node;
      node = document.createElementNS(xsl_namespace, "param");
      node.setAttribute("name", name);
      this.add_content(node, content);
      return node;
    },
    
    with_param: function (name, content)
    {
      var node;
      node = document.createElementNS(xsl_namespace, "with-param");
      node.setAttribute("name", name);
      this.add_content(node, content);
      return node;
    },
    
    element: function (name, content)
    {
      var node;
      node = document.createElementNS(xsl_namespace, "element");
      node.setAttribute("name", name);
      this.add_content(node, content);
      return node;
    },
    
    attribute: function (name, content)
    {
      var node;
      node = document.createElementNS(xsl_namespace, "attribute");
      node.setAttribute("name", name);
      this.add_content(node, content);
      return node;
    },
    
    text: function (content)
    {
      var node;
      node = document.createElementNS(xsl_namespace, "text");
      this.add_content(node, content);
      return node;
    },
    
    add_content: function (node, content)
    {
      if (!content) return;
      
      try
        {
          switch (typeof content)
            {
            case "object":
              for (var i = 0; i < content.length; ++i)
                {
                  node.appendChild(content[i]);
                }
              break;
            case "string":
              node.appendChild(document.createTextNode(content));
              break;
            default:
              node.appendChild(document.createTextNode(content.toString()));
            }
        }
      catch (exception)
        {
          application.message("Error: " + content.constructor
                              + ": " + exception);
        }
    }
  }
