<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:om="http://www.openmath.org/OpenMath"
  xmlns:transformiix="http://www.mozilla.org/TransforMiix"
  xmlns:a="http://www.matracas.org/ns/algebra"
  version="1.0">
  
  <xsl:output method="xml"/>
  
  <xsl:template match="/">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="*">
    <xsl:copy><xsl:copy-of select="@*"/><xsl:apply-templates/></xsl:copy>
  </xsl:template>
  
  <xsl:template match="a:app">
    <xsl:element name="xsl:template">
      <xsl:attribute name="match">
        <xsl:text>om:OMA[om:*[1][self::om:OMS][@cd='</xsl:text>
        <xsl:value-of select="@cd"/>
        <xsl:text>' and @name='</xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>']]</xsl:text>
        <xsl:if test="@cond">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="@cond"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
      </xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="a:rule[*[1][self::om:OMA][om:*[1][self::om:OMS]]]">
    <xsl:element name="xsl:template">
      <xsl:attribute name="match">
        <xsl:text>om:OMA[om:*[1][self::om:OMS][@cd='</xsl:text>
        <xsl:value-of select="om:OMA[1]/om:OMS[1]/@cd"/>
        <xsl:text>' and @name='</xsl:text>
        <xsl:value-of select="om:OMA[1]/om:OMS[1]/@name"/>
        <xsl:text>']]</xsl:text>
        <xsl:if test="@cond">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="@cond"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
      </xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <!-- match attributes for property templates: -->
  <xsl:template name="property_match_attribute_conmutative">
    <xsl:param name="lhs"/>
    <xsl:attribute name="match">
      <xsl:choose>
        <xsl:when test="$lhs[self::om:OMA]/*[1][self::om:OMS]">
          <xsl:text>om:OMA[om:*[1][self::om:OMS][@cd='</xsl:text>
          <xsl:value-of select="$lhs/om:OMS[1]/@cd"/>
          <xsl:text>' and @name='</xsl:text>
          <xsl:value-of select="$lhs/om:OMS[1]/@name"/>
          <xsl:text>']]</xsl:text>
          <xsl:apply-templates mode="build_cond" select="$lhs/om:*[local-name() != 'OMV']"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>
  
  <xsl:template name="property_match_attribute_not_conmutative">
    <xsl:param name="lhs"/>
    <xsl:attribute name="match">
      <xsl:choose>
        <xsl:when test="$lhs[self::om:OMA]/*[1][self::om:OMS]">
          <xsl:text>om:OMA[count(om:*) = </xsl:text>
          <xsl:value-of select="count($lhs/om:*)"/>
          <xsl:text>][om:*[1][self::om:OMS][@cd='</xsl:text>
          <xsl:value-of select="$lhs/om:OMS[1]/@cd"/>
          <xsl:text>' and @name='</xsl:text>
          <xsl:value-of select="$lhs/om:OMS[1]/@name"/>
          <xsl:text>']]</xsl:text>
          <xsl:apply-templates mode="build_cond_not_conmutative_not_n-ary" select="$lhs/om:*[local-name() != 'OMV']"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>
  
  <xsl:template match="a:property[@conmutative = 'true' and @n-ary = 'true'][*[1][self::om:OMA]/*[1][self::om:OMS][@cd='relation1' and @name='eq']]">
    <xsl:variable name="lhs" select="om:OMA[1]/*[2]"/>
    <xsl:variable name="rhs" select="om:OMA[1]/*[3]"/>
    <xsl:element name="xsl:template">
      <xsl:call-template name="property_match_attribute_conmutative">
	<xsl:with-param name="lhs" select="$lhs"/>
      </xsl:call-template>
      <xsl:variable name="inv_cond">
        <xsl:apply-templates mode="build_inv_cond" select="$lhs/om:*[local-name() != 'OMV']"/>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="$inv_cond">
          <xsl:element name="xsl:variable">
            <xsl:attribute name="name">untouched</xsl:attribute>
            <xsl:attribute name="select">
              <xsl:text>om:*[position() &gt; 1 and (</xsl:text>
              <xsl:value-of select="$inv_cond"/>
              <xsl:text>)]</xsl:text>
            </xsl:attribute>
          </xsl:element>
          <xsl:element name="xsl:choose">
            <xsl:element name="xsl:when">
              <xsl:attribute name="test">$untouched</xsl:attribute>
              <xsl:element name="xsl:copy">
                <xsl:element name="xsl:apply-templates">
                  <xsl:attribute name="select">$untouched</xsl:attribute>
                </xsl:element>
              </xsl:element>
            </xsl:element>
            <xsl:element name="xsl:otherwise">
              <xsl:element name="xsl:apply-templates">
                <xsl:attribute name="select">om:*[2]</xsl:attribute>
              </xsl:element>
            </xsl:element>
          </xsl:element>
        </xsl:when>
        <xsl:otherwise>
          <xsl:element name="xsl:copy">
            <xsl:element name="xsl:apply-templates"/>
          </xsl:element>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="a:property[@conmutative = 'true' and not(@n-ary = 'true')][*[1][self::om:OMA]/*[1][self::om:OMS][@cd='relation1' and @name='eq']]">
    <xsl:variable name="lhs" select="om:OMA[1]/*[2]"/>
    <xsl:variable name="rhs" select="om:OMA[1]/*[3]"/>
    <xsl:element name="xsl:template">
      <xsl:call-template name="property_match_attribute_conmutative">
	<xsl:with-param name="lhs" select="$lhs"/>
      </xsl:call-template>
      <xsl:variable name="inv_cond">
        <xsl:apply-templates mode="build_inv_cond" select="$lhs/om:*[local-name() != 'OMV']"/>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="$inv_cond">
          <xsl:element name="xsl:variable">
            <xsl:attribute name="name">untouched</xsl:attribute>
            <xsl:attribute name="select">
              <xsl:text>om:*[position() &gt; 1 and (</xsl:text>
              <xsl:value-of select="$inv_cond"/>
              <xsl:text>)]</xsl:text>
            </xsl:attribute>
          </xsl:element>
          <xsl:element name="xsl:choose">
            <xsl:element name="xsl:when">
              <xsl:attribute name="test">$untouched</xsl:attribute>
              <xsl:element name="xsl:copy">
                <xsl:element name="xsl:apply-templates">
                  <xsl:attribute name="select">$untouched</xsl:attribute>
                </xsl:element>
              </xsl:element>
            </xsl:element>
            <xsl:element name="xsl:otherwise">
              <xsl:element name="xsl:apply-templates">
                <xsl:attribute name="select">om:*[2]</xsl:attribute>
              </xsl:element>
            </xsl:element>
          </xsl:element>
        </xsl:when>
        <xsl:otherwise>
          <xsl:element name="xsl:copy">
            <xsl:element name="xsl:apply-templates"/>
          </xsl:element>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:for-each select="$rhs">
        <xsl:choose>
          <xsl:when test="self::om:OMV">
            
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="."/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="a:property[not(@conmutative = 'true') and not(@n-ary = 'true')][*[1][self::om:OMA]/*[1][self::om:OMS][@cd='relation1' and @name='eq']]">
    <xsl:variable name="lhs" select="om:OMA[1]/*[2]"/>
    <xsl:variable name="rhs" select="om:OMA[1]/*[3]"/>
    <xsl:element name="xsl:template">
      <xsl:call-template name="property_match_attribute_not_conmutative">
	<xsl:with-param name="lhs" select="$lhs"/>
      </xsl:call-template>
      <xsl:for-each select="$rhs">
        <xsl:choose>
          <xsl:when test="self::om:OMV">
            <xsl:variable name="variable_name" select="@name"/>
            <xsl:element name="xsl:apply-templates">
              <xsl:attribute name="select">
                <xsl:text>om:*[</xsl:text>
                <xsl:value-of select="count($lhs/om:OMV[@name = $variable_name]/preceding-sibling::om:*) + 1"/>
                <xsl:text>]</xsl:text>
              </xsl:attribute>
            </xsl:element>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="."/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>
  
  <xsl:template mode="build_cond" match="om:OMI">
    <xsl:text>[om:OMI[text() = '</xsl:text>
    <xsl:value-of select="text()"/>
    <xsl:text>']]</xsl:text>
  </xsl:template>
  
  <xsl:template mode="build_cond" match="om:OMV"/>
  
  <xsl:template mode="build_cond_not_conmutative_not_n-ary" match="om:OMI">
    <xsl:text>[om:*[</xsl:text>
    <xsl:value-of select="count(preceding-sibling::om:*) + 1"/>
    <xsl:text>][self::om:OMI[text() = '</xsl:text>
    <xsl:value-of select="text()"/>
    <xsl:text>']]]</xsl:text>
  </xsl:template>
  
  <xsl:template mode="build_cond_not_conmutative_not_n-ary" match="om:OMV"/>
  
  <xsl:template mode="build_inv_cond" match="om:OMI">
    <xsl:text>local-name() != OMI or not(text() = '</xsl:text>
    <xsl:value-of select="text()"/>
    <xsl:text>')</xsl:text>
  </xsl:template>
  
  <xsl:template mode="build_inv_cond" match="om:OMV"/>
  
</xsl:stylesheet>
