<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:om="http://www.openmath.org/OpenMath"
  xmlns:m="http://www.w3.org/1998/Math/MathML"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns="http://www.w3.org/1999/xhtml"
  version="1.0">
  
  <!-- xsl:include href="o2ht_om_oms.xsl"/ -->
  
  <xsl:output method="xml"/>

<!-- ============================================================ -->
<!-- ==================        OpenMath        ================== --> 
<!-- ============================================================ -->

  <xsl:template match="om:OMOBJ">
    <span class="OMOBJ">
      <m:math>
        <xsl:apply-templates/>
      </m:math>
    </span>
  </xsl:template>
  
  <xsl:template match="om:OMR[@xlink:href]">
    <xsl:variable name="href" select="@xlink:href"/>
    <m:mfenced separators=":" open="[" close="]">
      <xsl:call-template name="intradocument_href">
        <xsl:with-param name="class">_attr_href</xsl:with-param>
        <xsl:with-param name="href" select="$href"/>
      </xsl:call-template>
      <m:mspace width="0.4em"/>
      <xsl:apply-templates select="//om:*[@xml:id=$href]"/>
    </m:mfenced>
  </xsl:template>
  
  <xsl:template name="infix_arguments">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:for-each select="*">
      <xsl:choose>
        <xsl:when test="position()=1"></xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select=".">
            <xsl:with-param name="parent_precedence" select="$precedence"/>
          </xsl:apply-templates>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="not(position()=1 or (position()=last()))">
        <xsl:apply-templates select="../*[1]"/>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="prefix_arguments">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:apply-templates select="*[position() &gt; 1]">
      <xsl:with-param name="parent_precedence" select="$precedence"/>
    </xsl:apply-templates>
  </xsl:template>
  
  <xsl:template name="infix_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:element name="m:mfenced">
      <xsl:attribute name="separators"/>
      <xsl:if test="@xml:id">
        <xsl:attribute name="xml:id"><xsl:value-of select="@xml:id"/></xsl:attribute>
      </xsl:if>
      <xsl:attribute name="fixity">infix</xsl:attribute>
      <xsl:if test="not($parent_precedence &gt;= $precedence)">
        <xsl:attribute name="open"/>
        <xsl:attribute name="close"/>
      </xsl:if>
      <xsl:call-template name="infix_arguments">
        <xsl:with-param name="precedence" select="$precedence"/>
        <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
      </xsl:call-template>
    </xsl:element>
  </xsl:template>

  <xsl:template name="superindex_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <m:msup>
      <m:mrow>
        <xsl:apply-templates select="*[2]">
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:apply-templates>
      </m:mrow>
      <m:mrow>
        <xsl:apply-templates select="*[3]">
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:apply-templates>
      </m:mrow>
    </m:msup>
  </xsl:template>

  <xsl:template name="root_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <m:mroot>
      <xsl:call-template name="prefix_arguments">
        <xsl:with-param name="precedence" select="$precedence"/>
        <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
      </xsl:call-template>
    </m:mroot>
  </xsl:template>

  <xsl:template name="function_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <m:mrow>
      <xsl:apply-templates select="*[1]"/>
      <xsl:choose>
        <xsl:when test="count(*) &gt; 2 or om:OMA or $parent_precedence &gt; $precedence">
          <m:mfenced separators=",">
            <xsl:call-template name="prefix_arguments">
              <xsl:with-param name="precedence" select="$precedence"/>
              <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
            </xsl:call-template>
          </m:mfenced>
        </xsl:when>
        <xsl:otherwise>
	  <m:mspace width="0.3em"/>
          <xsl:call-template name="prefix_arguments">
            <xsl:with-param name="precedence" select="$precedence"/>
            <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </m:mrow>
  </xsl:template>
  
  <xsl:template name="prefix_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <m:mfenced separators="">
          <xsl:apply-templates select="*[1]"/>
          <xsl:apply-templates select="*[2]"/>
        </m:mfenced>
      </xsl:when>
      <xsl:otherwise>
        <m:mrow>
          <xsl:apply-templates select="*[1]"/>
          <xsl:apply-templates select="*[2]"/>
        </m:mrow>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="suffix_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <m:mfenced separators="">
          <xsl:apply-templates select="*[2]"/>
          <xsl:apply-templates select="*[1]"/>
        </m:mfenced>
      </xsl:when>
      <xsl:otherwise>
        <m:mrow>
          <xsl:apply-templates select="*[2]"/>
          <xsl:apply-templates select="*[1]"/>
        </m:mrow>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="sumatory_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <m:mfenced separators="" open="(" close=")">
          <m:munderover>
            <xsl:apply-templates select="*[1]"/>
            <!-- (oma interval1.*interval a b) -->
            <m:mrow>
              <xsl:apply-templates select="*[3]/*[2]/*"/>
              <m:mo>=</m:mo>
              <xsl:apply-templates select="*[2]/*[2]"/>
            </m:mrow>
            <xsl:apply-templates select="*[2]/*[3]"/>
          </m:munderover>
          <xsl:apply-templates select="*[3]/*[3]"/>
        </m:mfenced>
      </xsl:when>
      <xsl:otherwise>
        <m:munderover>
          <xsl:apply-templates select="*[1]"/>
          <!-- (oma interval1.*interval a b) -->
          <m:mrow>
            <xsl:apply-templates select="*[3]/*[2]/*"/>
            <m:mo>=</m:mo>
            <xsl:apply-templates select="*[2]/*[2]"/>
          </m:mrow>
          <xsl:apply-templates select="*[2]/*[3]"/>
        </m:munderover>
        <xsl:apply-templates select="*[3]/*[3]"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="fenced_notation">
    <xsl:param name="open">(</xsl:param>
    <xsl:param name="close">)</xsl:param>
    <xsl:param name="separators">,</xsl:param>
    <m:mfenced open="{$open}" close="{$close}" separators="{$separators}">
      <xsl:apply-templates select="*[position() &gt; 1]"/>
    </m:mfenced>
  </xsl:template>
  
  <xsl:template name="fraction_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <m:mfenced separators="" open="(" close=")">
          <m:mfrac>
            <xsl:for-each select="*">
              <xsl:choose>
                <xsl:when test="position()=1"></xsl:when>
                <xsl:otherwise>
                  <xsl:apply-templates select="."/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:for-each>
          </m:mfrac>
        </m:mfenced>
      </xsl:when>
      <xsl:otherwise>
        <m:mfrac>
          <xsl:for-each select="*">
            <xsl:choose>
              <xsl:when test="position()=1"></xsl:when>
              <xsl:otherwise>
                <xsl:apply-templates select="."/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:for-each>
        </m:mfrac>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="rational_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <m:mfenced separators="">
          <m:mn>
            <m:mfrac>
              <xsl:apply-templates select="*[2]"/>
              <xsl:apply-templates select="*[3]"/>
            </m:mfrac>
          </m:mn>
        </m:mfenced>
      </xsl:when>
      <xsl:otherwise>
        <m:mn>
          <m:mfrac>
            <xsl:apply-templates select="*[2]"/>
            <xsl:apply-templates select="*[3]"/>
          </m:mfrac>
        </m:mn>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="om:OMA">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:call-template name="function_notation">
      <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="om:OMA[*[1][@cd='org.matracas.algebra' and @name='sqrt']]">
    <m:msqrt><xsl:apply-templates select="om:*[2]"/></m:msqrt>
  </xsl:template>
  
  <xsl:template match="om:OMA[*[1]/@cd='arith1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='abs'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">|</xsl:with-param>
          <xsl:with-param name="close">|</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='divide'">
        <xsl:call-template name="fraction_notation">
          <xsl:with-param name="precedence">60</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='minus'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">50</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='plus'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">50</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='power'">
        <xsl:call-template name="superindex_notation">
          <xsl:with-param name="precedence">70</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='product'">
        <xsl:call-template name="sumatory_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='root'">
        <xsl:call-template name="root_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='sum'">
        <xsl:call-template name="sumatory_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='times'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">60</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='unary_minus'">
        <xsl:call-template name="prefix_notation">
          <xsl:with-param name="precedence">80</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='interval1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='interval_oo'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">(</xsl:with-param>
          <xsl:with-param name="close">)</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_oc'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">(</xsl:with-param>
          <xsl:with-param name="close">]</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_co'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">[</xsl:with-param>
          <xsl:with-param name="close">)</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_cc'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">[</xsl:with-param>
          <xsl:with-param name="close">]</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='integer1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='factorial'">
        <xsl:call-template name="suffix_notation">
          <xsl:with-param name="precedence">90</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='integer_interval'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">(</xsl:with-param>
          <xsl:with-param name="close">)</xsl:with-param>
          <xsl:with-param name="separators">&#x2026;</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">(</xsl:with-param>
          <xsl:with-param name="close">)</xsl:with-param>
          <xsl:with-param name="separators">&#x2026;</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_cc'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">[</xsl:with-param>
          <xsl:with-param name="close">]</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_co'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">[</xsl:with-param>
          <xsl:with-param name="close">)</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_oc'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">(</xsl:with-param>
          <xsl:with-param name="close">]</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_oo'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">(</xsl:with-param>
          <xsl:with-param name="close">)</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='logic1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='and'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">11</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='equals'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">5</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='implies'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">5</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='or'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">10</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='not'">
        <xsl:call-template name="prefix_notation">
          <xsl:with-param name="precedence">12</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='nums1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='rational'">
        <xsl:call-template name="rational_notation">
          <xsl:with-param name="precedence">65</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='relation1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='eq'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='geq'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='leq'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='gt'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='lt'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='simpletypes']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='funtype'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">11</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="om:OMA[*[1]/@cd='list1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='list'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">[</xsl:with-param>
          <xsl:with-param name="close">]</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="om:OMA[*[1]/@cd='set1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='cartesian_product'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">60</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='in'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">44</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='intersect'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">46</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='map'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">50</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='notin'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">44</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='notsubset'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">45</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='prsubset'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">45</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='set'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">{</xsl:with-param>
          <xsl:with-param name="close">}</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='setdiff'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">50</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='subset'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">45</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='suchthat'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">10</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='union'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">45</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMBIND">
    <m:mrow>
      <xsl:apply-templates select="*[1]"/>
      <xsl:apply-templates select="*[2]"/>
      <m:mo>.</m:mo>
      <m:mo>(</m:mo><xsl:apply-templates select="*[3]"/><m:mo>)</m:mo>
    </m:mrow>
  </xsl:template>
  
  <xsl:template match="om:OMBVAR">
    <m:mfenced open="[" close="]" separators="&#x2002;">
      <xsl:apply-templates/>
    </m:mfenced>
  </xsl:template>
  
  <xsl:template match="om:OMATTR">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="om:OMATP">
    <m:mo>{</m:mo><xsl:apply-templates/><m:mo>}</m:mo>
  </xsl:template>
  
  <xsl:template match="om:OMATP[om:OMS[@cd='simpletypes' and @name='type']]">
    <mfenced open="&#9001;" close="&#9002;" class="_math_type"><xsl:apply-templates select="*[position()!=1]"/></mfenced>
  </xsl:template>
  
  <xsl:template match="om:OMI">
    <m:mn><xsl:value-of select="."/></m:mn>
  </xsl:template>

  <xsl:template match="om:OMF">
    <xsl:choose>
      <xsl:when test="@dec">
        <m:mn><xsl:value-of select="@dec"/></m:mn>
      </xsl:when>
      <xsl:when test="@hex">
        <m:mn>#x<xsl:value-of select="@hex"/></m:mn>
      </xsl:when>
      <xsl:otherwise>
        <m:mn>NaN</m:mn>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMV">
    <m:mi><xsl:value-of select="@name"/></m:mi>
  </xsl:template>
  
  <xsl:template match="om:OMR">
    <xsl:call-template name="intradocument_href">
      <xsl:with-param name="href" select="@href"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="om:OMSTR">
    <m:mtext><xsl:apply-templates/></m:mtext>
  </xsl:template>

  <xsl:template match="om:OME">
    <!-- m:merror><xsl:apply-templates/></m:merror -->
    <m:mfenced open="[" close="]"><xsl:apply-templates/></m:mfenced>
  </xsl:template>
  
  <xsl:template match="om:OMR[@xlink:href]">
    <xsl:variable name="href" select="@xlink:href"/>
    <xsl:apply-templates select="//om:*[@xml:id=$href]"/>
  </xsl:template>
  
</xsl:stylesheet>
