///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2004,2005,2006,2007,2008,2009 Alberto González Palomo
//   Author: Alberto González Palomo <http://www.matracas.org>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////

var timer =
  {
  ms: 0,
  start_ms: 0,
  start: function() { this.start_ms = (new Date()).getTime(); },
  stop: function() { this.ms = (new Date()).getTime() - this.start_ms; }
  };

function QMath(base_url)
{
  var self = this;
  // Public functions:
  this.rebuild_context = rebuild_context;
  this.declare_variable = declare_variable;
  this.linear_to_openmath = linear_to_openmath;
  this.linear_to_parse_tree = linear_to_parse_tree;
  this.parse_tree_to_openmath = parse_tree_to_openmath;
  this.openmath_to_linear = openmath_to_linear;
  this.openmath_to_parse_tree = openmath_to_parse_tree;
  this.parse_tree_to_linear = parse_tree_to_linear;
  this.measure_performance = measure_performance;

  var openmath_namespace = "http://www.openmath.org/OpenMath";
  var    qmath_namespace = "http://www.matracas.org/ns/qmath";
  var   mathml_namespace = "http://www.w3.org/1998/Math/MathML";
  var    xhtml_namespace = "http://www.w3.org/1999/xhtml";

  var namespace_prefix_map =
    {
      "om"  : openmath_namespace,
      "q"   :    qmath_namespace,
      "m"   :   mathml_namespace,
      "h"   :    xhtml_namespace
    };

  var xml_serializer = new XMLSerializer();
  var gClipboardHelper;
  if ("Components" in window)
    {
      try
        {
          gClipboardHelper =
            Components.classes["@mozilla.org/widget/clipboardhelper;1"]
            .getService(Components.interfaces.nsIClipboardHelper);
        }
      catch (exception)
        {
          // This happens when using QMath in a web page, outside Sentido.
          // We just don't copy anything to the clipboard in that case.
          gClipboardHelper = null;
        }
    }

  var parser;
  var transformer_parse;
  var transformer_unparse;
  var stylesheet_unparse;
  var stylesheet_parse;
  var linear_as_xml;

  var transformer_build_stylesheet_parse;
  var debug = false;

  // Constructor code:
  if (! base_url) base_url = "";// Make sure it's a string, not "undefined".
  else if (! base_url.match(/\/$/)) base_url += "/";
  linear_as_xml = false;
  //parser = new Parser();
  //initialize_parser(parser);
  register_namespace_map(namespace_prefix_map);
  transformer_build_stylesheet_parse = new XSLTProcessor();
  var stylesheet;
  stylesheet = load_xml(base_url + "context_to_parse_stylesheet.xsl");
  if (!stylesheet) message("Can't load file: " + base_url + "context_to_parse_stylesheet.xsl");
  transformer_build_stylesheet_parse.importStylesheet(stylesheet);

  function grammar_action_declare_variable(parser, rule, matched_input)
  {
    parser.term_class("object_variable", [matched_input]);
    // TODO: We need to tell the parser to re-tokenize.

    return parser.symbol(matched_input);
  }

  function initialize_parser(p)
  {
    var t = p.token_table;
    var symbol = p.symbol;
    var rule = p.rule;
    var action_rule = p.action_rule;
    var term_class = p.term_class;
    var complement_term_class = p.complement_term_class;
    var reserve = p.reserve;

    var space = "\\s*";
    //p.ignore_space = true;
    rule("line_separator", ";");
    term_class("string_open",  ['"']);
    term_class("string_close", ['"']);
    complement_term_class("string_content", [t.string_close]);
    rule("string", t.string_open + p.any(t.string_content) + t.string_close);

    reserve("num_dec");
    reserve("num_int");
    reserve("num_rat");
    reserve("num_com");
    reserve("group_object");
    reserve("tuple_object");
    reserve("juxt_app_type_oo_o");
    reserve("juxt_app_type_mm_m");
    reserve("op_fact_app");
    reserve("op_exp_app");
    reserve("op_not_app");
    reserve("op_prod_app");
    reserve("op_plus_app");
    reserve("op_plus_app_unary");
    reserve("op_func_app");
    reserve("op_interval_app");
    reserve("op_eq_app");
    reserve("op_and_app");
    reserve("op_or_app");
    reserve("op_impl_app");
    reserve("binding_app");
    reserve("openmath_OMA");
    reserve("openmath_OMS");
    reserve("openmath_OMR");
    reserve("openmath_OMATTR");
    reserve("openmath_OMATP");

    /////////////////////////////////////////////////////////////////////
    // Classes:

    term_class("object", []);
    term_class("object_morphism", []);
    term_class("object_morphism_morphism", []);
    term_class("object_variable", []);
    term_class("object_morphism_variable", []);
    term_class("object_morphism_morphism_variable", []);
    term_class("op_fact", []);
    term_class("op_exp", []);
    term_class("op_prod", []);
    term_class("op_plus", []);
    term_class("op_interval", []);
    term_class("op_eq", []);
    term_class("op_not", []);
    term_class("op_and", []);
    term_class("op_or", []);
    term_class("op_impl", []);
    term_class("binding", []);

    term_class("morphism_app",
               [t.op_fact_app,
                t.op_exp_app,
                t.op_not_app,
                t.op_prod_app,
                t.juxt_app_type_oo_o,
                t.juxt_app_type_mm_m,
                t.op_plus_app,
                t.op_func_app,// This is the same as juxt_app_type_mo_o
                t.op_interval_app,
                t.op_eq_app,
                t.op_and_app,
                t.op_or_app,
                t.op_impl_app]
               );
    term_class("object",
               [t.string,
                t.num_int, t.num_dec, t.num_rat, t.num_com,
                t.tuple_object,
                t.group_object,
                t.morphism_app,
                //t.op_plus_app_unary,
                t.binding_app,
                t.object_variable,
                t.object_morphism_variable,
                t.object_morphism_morphism_variable,
		t.openmath_OMA, t.openmath_OMS, t.openmath_OMR, t.openmath_OMATTR, t.openmath_OMATP]
               );

    /////////////////////////////////////////////////////////////////////
    // Rules:

    ////////// Mathematics:
    //term_class("digit", ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]);
    term_class("decimal_separator", ["."]);
    term_class("list_separator", [","]);
    //rule("num_dec", p.any(t.digit) + t.decimal_separator + p.any(t.digit)    );
    //rule("num_rat", p.any(t.digit) + symbol("/") + p.any(t.digit)            );
    //rule("num_com", p.any(t.digit) + t.op_plus + p.any(t.digit) + symbol("i"));
    //rule("num_int", p.any(t.digit)                                           );
    rule("num_dec", "[0-9][0-9 ]*" + t.decimal_separator + "[0-9][0-9 ]*");
    //rule("num_rat", "[0-9][0-9 ]*" + symbol("/") + "[0-9][0-9 ]*");
    //rule("num_com", "[0-9][0-9 ]*" + t.op_plus + "[0-9][0-9 ]*" + symbol("i"));
    rule("num_int", "[0-9][0-9 ]*");

    rule("group_object_parenthesis",
         p.or([symbol("(") + space + t.object + space + symbol(")"),
               p.input_begin() + space + t.object + space + symbol(")"),
               symbol("(")     + space + t.object + space + p.input_end()])
         );
    rule("group_object_brackets",
         p.or([symbol("[") + space + t.object + space + symbol("]"),
               p.input_begin() + space + t.object + space + symbol("]"),
               symbol("[")     + space + t.object + space + p.input_end()])
         );
    rule("group_object_braces",
         p.or([symbol("{") + space + t.object + space + symbol("}"),
               p.input_begin() + space + t.object + space + symbol("}"),
               symbol("{")     + space + t.object + space + p.input_end()])
         );

    p.term_class("group_object",
                 [t["group_object_parenthesis"],
                  t["group_object_brackets"],
                  t["group_object_braces"]
                  ]);

    rule("openmath_OMA",
	 symbol("OMA") + t["group_object_parenthesis"]
	 );
    rule("openmath_OMS",
	 symbol("OMS") + t["group_object_parenthesis"]
	 );
    rule("openmath_OMR",
	 symbol("OMR") + t["group_object_parenthesis"]
	 );
    rule("openmath_OMATTR",
	 symbol("OMATTR") + t["group_object_parenthesis"]
	 );
    rule("openmath_OMATP",
	 symbol("OMATP") + t["group_object_parenthesis"]
	 );
    rule("op_func_app",
         p.or([t.object_morphism_variable + t.object,
               t.object_morphism + space + t.object
               ])
         );
    rule("op_fact_app",
         p.any(space + t.object + space + t.op_fact));
    rule("op_plus_app_unary_in_exponent",
         t.op_plus + t.object, [t.op_exp, ""]);
    rule("op_exp_app",
         t.object + p.any(space + t.op_exp + space + p.or([t.object, t.op_plus_app_unary_in_exponent])));
    rule("op_not_app",
         space + t.op_not + space + t.object);
    rule("op_prod_app",
         p.or([t.object + p.any(space + t.op_prod + space + t.object)])
         );
    term_class("multiplicative_object",
               [t.string,
                t.num_int, t.num_dec, t.num_rat, t.num_com,
                t.tuple_object,
                t.group_object,
                t.morphism_app,
                // In contrast to object, no op_plus_app_unary
                t.binding_app,
                t.object_variable,
                t.object_morphism_variable,
                t.object_morphism_morphism_variable]
               );
    rule("juxt_app_type_oo_o",
         t.object + p.any(space + t.object));
    rule("juxt_app_type_mm_m",
         t.morphism_app + p.any(space + t.morphism_app));
    rule("op_plus_app_unary",
         t.op_plus + t.object);
    rule("op_plus_app",// This is a special case: both prefix and infix.
         p.opt(t.object) + p.any(t.op_plus_app_unary));
    rule("op_interval_app",
         t.object + p.any(space + t.op_interval + space + t.object));
    rule("op_eq_app",
         t.object + p.any(space + t.op_eq + space + t.object));
    rule("op_and_app",
         t.object + p.any(space + t.op_and + space + t.object));
    rule("op_or_app",
         t.object + p.any(space + t.op_or + space + t.object));
    rule("op_impl_app",
         t.object + p.any(space + t.op_impl + space + t.object));
    rule("binding_app",
         t.binding + space + t.object + space + "." + space + t.object);
    //action_rule(grammar_action_declare_variable, "x", [t.binding, "."]);

    rule("tuple_object",
         t.object + space + p.any(symbol(",") + space + t.object)
         );
  }

  // context_url_array can be an array, or a single string.
  var current_context_url_array;
  function rebuild_context(context_url_array)
  {
    if (context_url_array) current_context_url_array = context_url_array;
    else                   context_url_array = current_context_url_array;

    try
      {
        parser = new Parser();
        initialize_parser(parser);
        stylesheet_parse = load_xml(base_url + "parse_tree_to_openmath.xsl");

        linear_as_xml = false;
        //stylesheet_unparse = load_xml(base_url + "openmath_to_linear.xsl");
        stylesheet_unparse = stylesheet_parse;

        if ("string" == typeof context_url_array)
          {
            load_context(parser,
                         stylesheet_parse, stylesheet_unparse,
                         context_url_array);
          }
        else
          {
            for (var i in context_url_array) load_context(parser,
                                                          stylesheet_parse,
                                                          stylesheet_unparse,
                                                          context_url_array[i]);
          }

        if (debug)
          {
            if (copy_to_clipboard(serialize_to_string(stylesheet_parse)))
	      {
		alert("templates from context copied to the clipboard");
	      }
          }
        //copy_to_clipboard(serialize_to_string(stylesheet_parse));
      }
    catch (exception)
      {
        message("Error: " + exception);
        stylesheet_parse   = null;
        stylesheet_unparse = null;
      }

    transformer_parse   = null;
    transformer_unparse = null;
    if (!stylesheet_parse || !stylesheet_unparse)
      {
        message("Error: could not build the stylesheet.");
        return;
      }

    try
      {
        transformer_parse = new XSLTProcessor();
        transformer_parse.importStylesheet(stylesheet_parse);
        transformer_unparse = transformer_parse;//new XSLTProcessor();
        //transformer_unparse.importStylesheet(stylesheet_unparse);
        //transformer_unparse = transformer_parse;
      }
    catch (exception)
      {
        message("Error: " + exception);
        copy_to_clipboard(serialize_to_string(stylesheet_parse));
      }
  }

  function declare_variable(name, type)
  {
    var symbol;
    symbol = parser.symbol(name);

    switch (type)
      {
      case "object":
        parser.term_class("object_variable", symbol);
        break;
      case "morphism":
        parser.term_class("object_morphism_variable", symbol);
        break;
      case "morphism_morphism":
        parser.term_class("object_morphism_morphism_variable", symbol);
        break;
      default:
        message("Unknown variable type: " + type);
      }
  }

  function linear_to_openmath(text)
  {
    var parse_tree;
    var omobj;
    parse_tree = linear_to_parse_tree(text);
    if (!parse_tree)
      {
        if (parser)
          {
            message("QMath::linear_to_openmath(" + text
                    + "): no result from parser");
          }
        else
          {
            message("QMath::linear_to_openmath(" + text + "): no parser");
          }
        omobj = null;
      }
    else
      {
        omobj = parse_tree_to_openmath(parse_tree);
        //var omobj;
        //if (xslt_result) omobj = xslt_result.documentElement;
        //else             omobj = null;
      }

    if (!omobj)
      {
        message("QMath::linear_to_openmath(" + text
                + "): no result from apply_transformer(transformer_parse, result)");
        omobj = document.createElementNS(openmath_namespace, "OMOBJ");
      }

    return omobj;
  }

  function linear_to_parse_tree(text)
  {
    var parse_tree;
    if (parser) parse_tree = parser.parse_to_XML(text);
    else        parse_tree = null;

    //if (parse_tree) message(serialize_to_string(parse_tree));
    //else            message("parse(" + text + "): there is no parse tree.");

    return parse_tree;
  }

  function parse_tree_to_openmath(parse_tree)
  {
    var omobj;
    omobj = apply_transformer(transformer_parse, parse_tree);
    move_range_attributes_to_properties(omobj);

    return omobj;
  }
  function move_range_attributes_to_properties(node)
  {
    if (node.nodeType != 1) return;
    if (node.hasAttribute("begin"))
      {
	node.begin = Number(node.getAttribute("begin"));
	node.removeAttribute("begin");
      }
    if (node.hasAttribute("end"))
      {
	node.end = Number(node.getAttribute("end"));
	node.removeAttribute("end");
      }
    node = node.firstChild;
    while (node)
      {
	move_range_attributes_to_properties(node);
	node = node.nextSibling;
      }
  }

  function openmath_to_linear(omobj)
  {
    return parse_tree_to_linear(openmath_to_parse_tree(omobj));
  }

  function openmath_to_parse_tree(omobj)
  {
    if (!omobj)
      {
        message("Warning: omobj is null.");
        return null;
      }

    var parse_tree;
    parse_tree = apply_transformer(transformer_unparse, omobj);
    /////copy_to_clipboard(serialize_to_string(stylesheet_unparse));
    /////message("XSLT unparse copied to clipboard!");
    //message(serialize_to_string(parse_tree));

    return parse_tree;
  }

  function parse_tree_to_linear(parse_tree)
  {
    return parser.unparse_from_XML(parse_tree);
  }

  function apply_transformer(transformer, tree)
  {
    var xslt_result;
    try
      {
        xslt_result = transformer.transformToDocument(tree).documentElement;
        xslt_result.nodeName;// To check that it's not null.
      }
    catch (exception)
      {
        var name;
        var stylesheet;
        if (transformer == transformer_parse)
          {
            name = "transformer_parse";
            stylesheet = stylesheet_parse;
          }
        else
          {
            name = "transformer_unparse";
            stylesheet = stylesheet_unparse;
          }
        message("Error: name = " + name + ": " + exception);
        xslt_result = null;

        var xml_content;
        xml_content = serialize_to_string(stylesheet);
        if (copy_to_clipboard(xml_content))
	  {
            message("Faulty XSLT stylesheet copied to clipboard.");
	  }
      }

    return xslt_result;
  }

  function load_xml(url)
  {
    var result;
    try
      {
        var request;
        request = new XMLHttpRequest();
        request.open("GET", url, false);
        request.send(null);
        result = request.responseXML;
      }
    catch (exception)
      {
        message("Error: " + url + ":\n" + exception);
      }

    return result;
  }

  function serialize_to_string(node)
  {
    return xml_serializer.serializeToString(node);
  }

  function copy_to_clipboard(text)
  {
    if (!gClipboardHelper) return false;
    gClipboardHelper.copyString(text);
    return true;
  }

  function load_context(parser, stylesheet_parse, stylesheet_unparse, url)
  {
    try
      {
        var context_document;
        context_document = load_xml(url);
        var templates_from_context;
        templates_from_context = transformer_build_stylesheet_parse.transformToFragment(context_document.documentElement, stylesheet_parse);
	//alert("Stylesheet from context:\n" + serialize_to_string(templates_from_context));
        stylesheet_parse.documentElement.appendChild(templates_from_context);
        var settings_iterator;
        settings_iterator = xpath('q:document/q:tokenizer-settings/@*',
                                  context_document,
                                  XPathResult.ORDERED_NODE_ITERATOR_TYPE);
        if (settings_iterator)
          {
            var tokenizer_setting;
            while ((tokenizer_setting = settings_iterator.iterateNext()))
              {
                switch (tokenizer_setting.name)
                  {
                  case "ignore-space":
                    parser.ignore_space = ("true" == tokenizer_setting.value);
                    break;
                  case "implicit-variable-name-pattern":
                    parser.implicit_variable_name_pattern = tokenizer_setting.value;
                    if (parser.implicit_variable_name_pattern
                        && '^' != parser.implicit_variable_name_pattern[0])
                      {
                        parser.implicit_variable_name_pattern =
                          "^(?:"
                          + parser.implicit_variable_name_pattern
                          + ")";
                      }
                    break;
                  default:
                    message("Error: unknown tokenizer setting '"
                            + tokenizer_setting.name + "'");
                  }
              }
          }

        var symbol_iterator;
        symbol_iterator = xpath('//q:symbol',
                                context_document,
                                XPathResult.ORDERED_NODE_ITERATOR_TYPE);
        if (symbol_iterator)
          {
            var term_classes;
            term_classes = {};
            var attributes;
            attributes = {};
            var attribute_names;
            attribute_names = ["token", "type", "cd", "name"];
            var op_plus_implicit_symbol_token;

            var symbol;
            while ((symbol = symbol_iterator.iterateNext()))
              {
                for (var i in attribute_names)
                  {
                    var name;
                    name = attribute_names[i];
                    if (symbol.hasAttribute(name))
                      {
                        attributes[name] = symbol.getAttribute(name);
                      }
                    else
                      {
                        var attribute_node;
                        attribute_node =
                          xpath_first("ancestor::q:group/@" + name, symbol);
                        if (attribute_node)
                          {
                            attributes[name] = attribute_node.value;
                          }
                        else switch (attributes["type"])
                          {
                          case "object_variable":
                          case "object_morphism_variable":
                          case "object_morphism_morphism_variable":
                            if ("cd" == name || "name" == name) break;
                            // Otherwise fall through:
                          default:
                            message("Error: " + url + ": missing attribute '"
                                    + name + "' at "
                                    + build_xpath(symbol)
                                    );
                          }
                      }
                  }

                if ("op_plus" == attributes.type
                    && (! (symbol.hasAttribute("unary_cd")
                           && symbol.hasAttribute("unary_name")
                           )
                        )
                    )
                  {
                    if (op_plus_implicit_symbol_token)
                      {
                        message("Error in context: " + url + "\n"
                                + "The symbol '" + attributes.token
                                + "' does not have an explicit unary variant, and there is already a symbol with an implicit unary variant: '"
                                + op_plus_implicit_symbol_token + "'");
                      }
                    else
                      {
                        op_plus_implicit_symbol_token = attributes.token;
                      }
                  }

                var token_array;
                token_array = term_classes[attributes.type];
                if (!token_array)
                  {
                    token_array = new Array();
                    term_classes[attributes.type] = token_array;
                  }
                token_array.push(attributes.token);
                // It's faster to collect all the tokens in each term class
                // and compile them together, because otherwise the rule
                // dependencies have to be recomputed each time, and the
                // affected rules re-compiled.
                //parser.term_class(attributes.type, [attributes.token]);
              }

            for (var type in term_classes)
              {
                parser.term_class(type, term_classes[type]);
              }
          }
      }
    catch (exception)
      {
        message("Error: url = '" + url + "': "
                + exception);
      }
  }

  function measure_performance(text)
  {
    if (!text) text = "413x^7-11x^6+8172x^5-311x^4+8873x^3-12x^2+1132x+892/112+(124/pi^66)+413x^7-11x^6+8172x^5-311x^4+8873x^3-12x^2+1132x+892/112+(124/pi^66)+413x^7-11x^6+8172x^5-311x^4+8873x^3-12x^2+1132x+892/112+(124/pi^66)+413x^7-11x^6+8172x^5-311x^4+8873x^3-12x^2+1132x+892/112+(124/pi^66)";
    var results;
    results = [];
    var timer;
    timer = new Progress_reporter();
    timer.begin_wait();
    var parse_tree;
    parse_tree = linear_to_parse_tree(text);
    timer.end_wait();
    results.push("linear_to_parse_tree(): " + timer.get_wait_length() + "ms");
    var new_omobj;
    timer.begin_wait();
    new_omobj = parse_tree_to_openmath(parse_tree);
    timer.end_wait();
    results.push("parse_tree_to_openmath(): " + timer.get_wait_length() + "ms");

    return results;
  }

}
