<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:om="http://www.openmath.org/OpenMath"
  xmlns:m="http://www.w3.org/1998/Math/MathML"
  xmlns="http://www.w3.org/1999/xhtml"
  version="1.0">
  
  <xsl:output method="text"/>
  <xsl:strip-space elements="*"/>
  
  <xsl:template match="/"><xsl:apply-templates/></xsl:template>
  
  <xsl:template match="*"/>
  <xsl:template match="om:*"/>
  
  <xsl:template match="om:OMOBJ">
    <xsl:apply-templates>
      <xsl:with-param name="parent_precedence" select="0"/>
    </xsl:apply-templates>
  </xsl:template>
  
  <xsl:template match="om:OMI">
    <xsl:value-of select="."/>
  </xsl:template>
  
  <xsl:template match="om:OMF[@dec]">
    <xsl:value-of select="@dec"/>
  </xsl:template>
  <xsl:template match="om:OMF[@hex]">
    <xsl:text>0x</xsl:text><xsl:value-of select="@dec"/>
  </xsl:template>
  
  <xsl:template match="om:OMS">
    <xsl:text>OMS('</xsl:text>
    <xsl:value-of select="@cd"/>
    <xsl:text>', '</xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>')</xsl:text>
  </xsl:template>
  
  <xsl:template match="om:OMV[@name]">
    <xsl:value-of select="@name"/>
  </xsl:template>
  
  <xsl:template match="om:OMR">
    <xsl:text>[#</xsl:text><xsl:value-of select="@href"/><xsl:text>]</xsl:text>
  </xsl:template>
  
  <xsl:template match="om:OMSTR">
    <xsl:text>"</xsl:text><xsl:value-of select="."/><xsl:text>"</xsl:text>
  </xsl:template>
  
  <xsl:template match="om:OME">
    <xsl:text>[ERROR: </xsl:text><xsl:apply-templates/><xsl:text>]</xsl:text>
  </xsl:template>
  
  <xsl:template match="om:OMA">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">100</xsl:param>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>(</xsl:text>
    <xsl:call-template name="separated_values">
      <xsl:with-param name="separator">, </xsl:with-param>
      <xsl:with-param name="parent_precedence" select="100"/>
    </xsl:call-template>
    <xsl:text>)</xsl:text>
  </xsl:template>
  
  <xsl:template match="om:OMATTR">
    <xsl:text>[@</xsl:text><xsl:apply-templates/><xsl:text>]</xsl:text>
  </xsl:template>
  
  <xsl:template match="om:OMATP">
    <xsl:text>&lt;</xsl:text>
    <xsl:if test="@xml:id"><xsl:attribute name="id"><xsl:value-of select="@xml:id"/></xsl:attribute></xsl:if>
    <xsl:call-template name="separated_values">
      <xsl:with-param name="separator">, </xsl:with-param>
      <xsl:with-param name="parent_precedence" select="100"/>
    </xsl:call-template>
    <xsl:text>&gt;</xsl:text>
  </xsl:template>
  
  <xsl:template match="om:OMBIND">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">100</xsl:param>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>(</xsl:text>
    <xsl:call-template name="separated_values">
      <xsl:with-param name="separator">, </xsl:with-param>
      <xsl:with-param name="parent_precedence" select="100"/>
    </xsl:call-template>
    <xsl:text>)</xsl:text>
  </xsl:template>
  
  <xsl:template match="om:OMBVAR">
    <xsl:param name="parent_precedence">100</xsl:param>
    <xsl:for-each select="*">
      <xsl:apply-templates select=".">
        <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
      </xsl:apply-templates>
      <xsl:if test="position() != last()">
        <xsl:text>, </xsl:text>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="om:OMA[om:OMS[@cd='simpletypes' and @name='funtype']]">
    <xsl:param name="parent_precedence">100</xsl:param>
    <xsl:call-template name="infix">
      <xsl:with-param name="symbol">&#x2192;</xsl:with-param>
      <xsl:with-param name="precedence">20</xsl:with-param>
      <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template name="prefix">
    <xsl:param name="symbol">?</xsl:param>
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">100</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <xsl:value-of select="$symbol"/>
        <xsl:text>(</xsl:text>
        <xsl:apply-templates select="*[position() &gt; 1]">
          <xsl:with-param name="parent_precedence" select="$precedence"/>
        </xsl:apply-templates>
        <xsl:text>)</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$symbol"/>
        <xsl:apply-templates select="*[position() &gt; 1]">
          <xsl:with-param name="parent_precedence" select="$precedence"/>
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="postfix">
    <xsl:param name="symbol">?</xsl:param>
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">100</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <xsl:text>(</xsl:text>
        <xsl:apply-templates select="*[position() &gt; 1]">
          <xsl:with-param name="parent_precedence" select="$precedence"/>
        </xsl:apply-templates>
        <xsl:text>)</xsl:text>
        <xsl:value-of select="$symbol"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="*[position() &gt; 1]">
          <xsl:with-param name="parent_precedence" select="$precedence"/>
        </xsl:apply-templates>
        <xsl:value-of select="$symbol"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="infix">
    <xsl:param name="symbol">?</xsl:param>
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">100</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <xsl:text>(</xsl:text>
        <xsl:call-template name="separated_values">
          <xsl:with-param name="separator">&#x200B;<xsl:value-of select="$symbol"/>&#x200B;</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$precedence"/>
        </xsl:call-template>
        <xsl:text>)</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="separated_values">
          <xsl:with-param name="separator">&#x200B;<xsl:value-of select="$symbol"/>&#x200B;</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="separated_values">
    <xsl:param name="separator">?</xsl:param>
    <xsl:param name="parent_precedence">90</xsl:param>
    <xsl:for-each select="*[position() &gt; 1]">
      <xsl:apply-templates select=".">
        <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
      </xsl:apply-templates>
      <xsl:if test="position() != last()">
        <xsl:value-of select="$separator"/>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  
</xsl:stylesheet>
