<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:c="http://www.matracas.org/ns/cascada" xmlns:om="http://www.openmath.org/OpenMath" xmlns:transformiix="http://www.mozilla.org/TransforMiix" xmlns="http://www.openmath.org/OpenMath" version="1.0">
  
  <xsl:output method="xml"/>
  
  <xsl:template match="/">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="c:result">
    <xsl:variable name="content" select="c:*[local-name() != 'line_separator']"/>
    <xsl:choose>
      <xsl:when test="$content">
        <xsl:for-each select="$content">
          <OMOBJ><xsl:apply-templates select="."/></OMOBJ>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <OMOBJ/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="OMOBJ">
    <c:result><xsl:apply-templates/></c:result>
  </xsl:template>
  
  <xsl:template match="transformiix:result">
    <xsl:apply-templates select="*[1]"/>
  </xsl:template>
  
  <xsl:template match="c:text">
    <OMSTR><xsl:value-of select="@literal"/></OMSTR>
  </xsl:template>
  
  <xsl:template match="c:group_object">
    <xsl:apply-templates select="c:*[position() mod 2 = 0]"/>
  </xsl:template>
  
  <xsl:template match="c:line_separator"/>
  
  <xsl:template match="c:unfenced_tuple_object">
    <OMA>
      <OMS cd="list1" name="list"/>
      <xsl:apply-templates select="c:*[position() mod 2 = 1]"/>
    </OMA>
  </xsl:template>
  
  <xsl:template match="c:tuple_object">
    <xsl:choose>
      <xsl:when test="parent::c:op_func_app">
        <xsl:apply-templates select="c:*[position() mod 2 = 0]"/>
      </xsl:when>
      <xsl:otherwise>
        <OMA>
          <OMS cd="list1" name="list"/>
          <xsl:apply-templates select="c:*[position() mod 2 = 0]"/>
        </OMA>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="c:set_object">
    <OMA>
      <OMS cd="set1" name="set"/>
      <xsl:apply-templates select="c:*[position() mod 2 = 0]"/>
    </OMA>
  </xsl:template>
  
  <xsl:template name="binding">
    <xsl:param name="list" select="c:*[local-name() != 'text' or @literal != ' ']"/>
    <OMBIND>
      <xsl:apply-templates select="$list[1]"/>
      <OMBVAR>
        <xsl:for-each select="$list[2]/*[local-name() != 'text' or @literal != ' ']">
          <xsl:apply-templates/>
        </xsl:for-each>
      </OMBVAR>
      <xsl:apply-templates select="$list[3]"/>
      <!-- We ignore any extra arguments. -->
    </OMBIND>
  </xsl:template>
  
  <xsl:template name="prefix">
    <xsl:param name="cd"/><xsl:param name="name"/>
    <OMA>
      <OMS cd="{$cd}" name="{$name}"/>
      <xsl:apply-templates select="*[2]"/><!-- Ignore spurious arguments. -->
    </OMA>
  </xsl:template>
  
  <xsl:template name="postfix">
    <xsl:param name="cd"/><xsl:param name="name"/>
    <OMA>
      <OMS cd="{$cd}" name="{$name}"/>
      <xsl:apply-templates select="*[2]"/><!-- Ignore spurious arguments. -->
    </OMA>
  </xsl:template>
  
  <xsl:template name="infix_prefix_left-to-right_n-ary">
    <xsl:param name="list" select="c:*[local-name() != 'text' or @literal != ' ']"/>
    <xsl:param name="begins_with_unary" select="true"/>
    <xsl:param name="unary_cd"/><xsl:param name="unary_name"/>
    <xsl:variable name="list_length" select="count($list)"/>
    <xsl:choose>
      <xsl:when test="$list_length = 1">
        <xsl:choose>
          <xsl:when test="$unary_cd and $unary_name">
            <OMA>
              <!-- This is the prefix unary case, with specific symbol. -->
              <OMS cd="{$unary_cd}" name="{$unary_name}"/>
              <xsl:choose>
                <xsl:when test="$begins_with_unary">
                  <xsl:apply-templates select="$list[1]/c:*[2]"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:apply-templates select="$list[1]"/>
                </xsl:otherwise>
              </xsl:choose>
            </OMA>
          </xsl:when>
          <xsl:otherwise>
            <!-- This is the prefix unary case, if unary application is
                 the identity, like in the case of arith1:plus. -->
            <xsl:choose>
              <xsl:when test="$begins_with_unary">
                <xsl:apply-templates select="$list[1]/c:*[2]"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:apply-templates select="$list[1]"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="$list_length = 2">
        <xsl:choose>
          <xsl:when test="$begins_with_unary">
            <OMA>
              <xsl:apply-templates select="$list[2]/c:*[1]"/>
              <xsl:call-template name="infix_prefix_left-to-right_n-ary">
                <xsl:with-param name="list" select="$list[1]"/>
                <xsl:with-param name="begins_with_unary" select="$begins_with_unary"/>
                <xsl:with-param name="unary_cd" select="$unary_cd"/>
                <xsl:with-param name="unary_name" select="$unary_name"/>
              </xsl:call-template>
              <xsl:apply-templates select="$list[2]/c:*[2]"/>
            </OMA>
          </xsl:when>
          <xsl:otherwise>
            <OMA>
              <xsl:apply-templates select="$list[2]/c:*[1]"/>
              <xsl:apply-templates select="$list[1]"/>
              <xsl:apply-templates select="$list[2]/c:*[2]"/>
            </OMA>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="last_operator_token" select="$list[last()]/c:*[1]/@literal"/>
        <xsl:variable name="last_different_operator" select="$list/c:*[1][@literal != $last_operator_token][last()]"/>
        <OMA>
          <xsl:apply-templates select="$list[last()]/c:*[1]"/>
          <xsl:choose>
            <xsl:when test="$last_different_operator">
              <xsl:variable name="position_of_last_different_operator" select="count($last_different_operator/../preceding-sibling::c:*) + 1"/>
              <xsl:call-template name="infix_prefix_left-to-right_n-ary">
                <xsl:with-param name="list" select="$list[position() &lt;= $position_of_last_different_operator]"/>
              </xsl:call-template>
              <xsl:apply-templates select="$list[position() &gt; $position_of_last_different_operator]/c:*[2]"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:choose>
                <xsl:when test="$begins_with_unary">
                  <xsl:apply-templates select="$list/c:*[2]"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:apply-templates select="$list[1]"/>
                  <xsl:apply-templates select="$list[position() &gt; 1]/c:*[2]"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:otherwise>
          </xsl:choose>
        </OMA>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="infix_left-to-right_n-ary">
    <xsl:param name="list" select="c:*[local-name() != 'text' or @literal != ' ']"/>
    <xsl:variable name="list_length" select="count($list)"/>
    <OMA>
      <xsl:choose>
        <xsl:when test="$list_length &lt; 3">
          <!-- An infix operator should have at least two arguments. -->
          <xsl:text>ERROR:infix operator has only </xsl:text><xsl:value-of select="count($list)-1"/><xsl:text> argument: [</xsl:text><xsl:apply-templates select="$list"/><xsl:text>]</xsl:text>
        </xsl:when>
        <xsl:when test="$list_length = 3">
          <xsl:apply-templates select="$list[2]"/>
          <xsl:apply-templates select="$list[1]"/>
          <xsl:apply-templates select="$list[3]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="operators" select="$list[position() mod 2 = 0]"/>
          <xsl:apply-templates select="$operators[last()]"/>
          <xsl:variable name="last_operator_token" select="$operators[last()]/@literal"/>
          <xsl:variable name="last_different_operator" select="$operators[@literal != $last_operator_token][last()]"/>
          <xsl:choose>
            <xsl:when test="$last_different_operator">
              <xsl:variable name="position_of_last_different_operator" select="count($last_different_operator/preceding-sibling::c:*) + 1"/>
              <xsl:variable name="position_of_first_argument_to_same_operator" select="$position_of_last_different_operator + 1"/>
              <xsl:call-template name="infix_left-to-right_n-ary">
                <xsl:with-param name="list" select="$list[position() &lt;= $position_of_first_argument_to_same_operator]"/>
              </xsl:call-template>
              <xsl:apply-templates select="$list[position() &gt; $position_of_first_argument_to_same_operator and position() mod 2 = 1]"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="$list[position() mod 2 = 1]"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </OMA>
  </xsl:template>
  
  <xsl:template name="infix_left-to-right">
    <xsl:param name="list" select="c:*[local-name() != 'text' or @literal != ' ']"/>
    <OMA>
      <xsl:choose>
        <xsl:when test="count($list) = 3">
          <xsl:apply-templates select="$list[2]"/>
          <xsl:apply-templates select="$list[1]"/>
          <xsl:apply-templates select="$list[3]"/>
        </xsl:when>
        <xsl:when test="count($list) &gt; 3">
          <xsl:apply-templates select="$list[last()-1]"/>
          <xsl:call-template name="infix_left-to-right">
            <xsl:with-param name="list" select="$list[position() &lt;= last()-2]"/>
          </xsl:call-template>
          <xsl:apply-templates select="$list[last()]"/>
        </xsl:when>
        <xsl:otherwise>
          <!-- An infix operator should have at least two arguments. -->
          <xsl:text>ERROR:infix operator has only </xsl:text><xsl:value-of select="count($list)-1"/><xsl:text> argument: [</xsl:text><xsl:apply-templates select="$list"/><xsl:text>]</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </OMA>
  </xsl:template>
  
  <xsl:template name="infix_right-to-left">
    <xsl:param name="list" select="c:*[local-name() != 'text' or @literal != ' ']"/>
    <OMA>
      <xsl:choose>
        <xsl:when test="count($list) = 3">
          <xsl:apply-templates select="$list[2]"/>
          <xsl:apply-templates select="$list[1]"/>
          <xsl:apply-templates select="$list[3]"/>
        </xsl:when>
        <xsl:when test="count($list) &gt; 3">
          <xsl:apply-templates select="$list[2]"/>
          <xsl:apply-templates select="$list[1]"/>
          <xsl:call-template name="infix_right-to-left">
            <xsl:with-param name="list" select="$list[position() &gt; 2]"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <!-- An infix operator should have at least two arguments. -->
          <xsl:text>ERROR:infix operator has only </xsl:text><xsl:value-of select="count($list) - 1"/><xsl:text> argument: [</xsl:text><xsl:apply-templates select="$list"/><xsl:text>]</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </OMA>
  </xsl:template>
  
  <!-- What to do with undefined tokens. -->
  <xsl:template match="c:token">
    <xsl:if test="@literal"><OMV name="{@literal}"/></xsl:if>
  </xsl:template>
  
  <xsl:template match="OMV">
    <c:token literal="{@name}"/>
  </xsl:template>
  
  <xsl:template match="c:token[@literal and translate(@literal, '0123456789', '') = '']">
    <xsl:value-of select="@literal"/>
  </xsl:template>
  
  <xsl:template match="c:num_int">
    <OMI><xsl:apply-templates/></OMI>
  </xsl:template>
  
  <xsl:template match="om:OMI">
    <c:num_int>
      <!-- TODO: output each digit wrapped in a digit node. -->
      <xsl:apply-templates/>
    </c:num_int>
  </xsl:template>
  
  <xsl:template match="c:num_dec">
    <OMF>
      <xsl:attribute name="dec">
        <xsl:for-each select="c:token">
          <xsl:value-of select="@literal"/>
        </xsl:for-each>
      </xsl:attribute>
    </OMF>
  </xsl:template>
  
  <xsl:template match="om:OMF">
    <c:num_dec>
      <!-- TODO: output each digit wrapped in a digit node. -->
      <xsl:value-of select="@dec"/>
    </c:num_dec>
  </xsl:template>
  
  <xsl:template match="c:num_rat">
    <!-- TODO: simplify this select since we don't have space tokens in the parse tree any more. -->
    <xsl:variable name="fraction_line_operator" select="c:*[@literal and translate(@literal, '0123456789', '')][1]"/>
    <OMA>
      <OMS cd="nums1" name="rational"/>
      <OMI><xsl:apply-templates select="$fraction_line_operator/preceding-sibling::c:*"/></OMI>
      <OMI><xsl:apply-templates select="$fraction_line_operator/following-sibling::c:*"/></OMI>
    </OMA>
  </xsl:template>
  
  <xsl:template match="OMA[*[1][self::OMS][@cd='nums1' and @name='rational']]">
    <c:num_rat>
      <xsl:apply-templates select="*[2]"/>
      <c:literal token="{*[1]}"/>
      <xsl:apply-templates select="*[3]"/>
    </c:num_rat>
  </xsl:template>
  
  <xsl:template match="OMA">
    <c:op_func_app>
      <c:token literal="OMA"/>
      <xsl:apply-templates/>
    </c:op_func_app>
  </xsl:template>
  
  <xsl:template match="OMS">
    <c:op_func_app>
      <c:token literal="OMS"/>
      <c:text literal="{@cd}"/>
      <c:text literal="{@name}"/>
    </c:op_func_app>
  </xsl:template>
  
  <xsl:template match="OMR">
    <c:op_func_app>
      <c:token literal="OMR"/>
      <c:text literal="{@href}"/>
    </c:op_func_app>
  </xsl:template>
  
  <!-- ========================= Defined symbols: ====================== -->
  
</xsl:stylesheet>
