///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2006 Alberto González Palomo
//   Author: Alberto González Palomo <http://www.matracas.org>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////

//const    xml_namespace = "http://www.w3.org/XML/1998/namespace";
//const mathml_namespace = "http://www.w3.org/1998/Math/MathML";
const algebra_namespace = "http://www.matracas.org/ns/algebra";

const gClipboardHelper = Components
  .classes["@mozilla.org/widget/clipboardhelper;1"]
  .getService(Components.interfaces.nsIClipboardHelper);

const xul_namespace = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
const xhtml_namespace = 'http://www.w3.org/1999/xhtml';

const namespace_prefix_map =
  {
    "xul":xul_namespace,
    "h":xhtml_namespace
  };

// This is almost the same function as in algebra.js, only taking now an
// argument more for the namespace, since we use XUL, XHTML, and MathML here.
function e(namespace, name, attributes, children)
{
  var element;
  element = document.createElementNS(namespace, name);
  if (attributes)
    for (var a in attributes) element.setAttribute(a, attributes[a]);
  if (children)
    for (var c in children)
      {
        if ("string" == typeof(children[c]))
          element.appendChild(document.createTextNode(children[c]));
        else
          element.appendChild(children[c]);
      }
  
  return element;
}

const serializer = new XMLSerializer();
var application;

var interface_style = "UNDEFINED";
var parser;// Read.
var kernel;// Eval.
var transformer_display;// Print.

var current_input;

var debug = false;

function init()
{
  window.setCursor("wait");
  application = window.parent;
  application.register_namespace_map(namespace_prefix_map);
  window.xpath        = application.xpath;
  window.xpath_first  = application.xpath_first;
  window.xpath_number = application.xpath_number;
  window.xpath_string = application.xpath_string;
  
  kernel = application.kernel;
  parser = application.new_parser();
  
  set_interface(interface_style);
  
  append_notebook_entry(1);
  current_input = get_input_text_field(1);
  current_input.focus();
  window.setCursor("default");
}

function append_notebook_entry(index)
{
  var notebook_entry;
  notebook_entry = new_notebook_entry(index);
  document.getElementById("notebook-body").appendChild(notebook_entry);
}

function set_interface(interface_style)
{
  parser.rebuild_context(["chrome://algebra/content/styles/" + interface_style + "/context.xml"]);
  transformer_display = new XSLTProcessor();
  transformer_display.importStylesheet(load_xml("formula.xsl"));
}

function load_xml(url)
{
  var result;
  try
    {
      var request;
      request = new XMLHttpRequest();
      request.open("GET", url, false);
      request.send(null);
      result = request.responseXML;
    }
  catch (exception)
    {
      message("Error: " + url + ":\n" + exception);
    }
  
  return result;
}

function new_input_xslt(index)
{
  var notebook_entry;
  notebook_entry = e(algebra_namespace, "notebook-entry");
  view_transformer.setParameter(null, "index", index);
  
  var fragment;
  fragment = view_transformer.transformToFragment(notebook_entry, document);
  
  return fragment;
}

function focus_input(input)
{
  current_input = input;
}

function blur_input(input)
{
  current_input = null;
}

function get_input_text_field(index)
{
  return document.getElementById("input" + index);
}

function get_output_node(index)
{
  var output;
  var input;
  input = get_input_text_field(index);
  if (input)
    {
      output = document.getElementById("output" + index);
      if (!output)
        {
          create_output_node_for_notebook_entry(input, index);
          output = document.getElementById("output" + index);
        }
    }
  
  return output;
}

function resize_input(input, lines)
{
  if (!lines) lines = input.value.replace(/./g, "").length + 1;
  input.setAttribute("rows", lines);
}

function update_notebook_entry(index, input)
{
  var output;
  output = get_output_node(index);
  if (!output)
    {
      message("No output " + index);
      return;
    }
  output.innerHTML = "";
  //while (output.lastChild) output.removeChild(output.lastChild);
  
  var expression;
  expression = parser.linear_to_openmath(input.value);
  if (debug)
    {
      if (parser.get_parse_result())
        {
          var parse_result;
          parse_result = serializer.serializeToString(parser.get_parse_result());
          output.appendChild(document.createTextNode(parse_result));
          output.appendChild(e(xhtml_namespace, "br"));
        }
    }
  if (!kernel)
    {
      output.appendChild(document.createTextNode("Error: no kernel"));
      return;
    }
  
  if ("OMOBJ" != expression.localName) expression = expression.firstChild;
  while (expression)
    {
      if (expression.previousSibling)
        {
          output.appendChild(e(xhtml_namespace, "hr"));
        }
      process_input_expression(expression, output);
      expression = expression.nextSibling;
    }
}

function process_input_expression(expression, output)
{
  var result;
  var result_string;
  result = expression;
  result_string = serializer.serializeToString(result);
  var results;
  results = new Object();
  var step;
  step = 0;
  do
    {
      if (step > 0)
        {
          output.appendChild(e(xhtml_namespace, "br"));
          output.appendChild(e(xhtml_namespace, "span",
                               {"class":"step-bullet"}
                               ));
          transformer_display.setParameter(algebra_namespace, "mark_changed",
                                           "true");
        }
      else
        {
          transformer_display.setParameter(algebra_namespace, "mark_changed",
                                           "false");
        }
      var displayed_formula;
      displayed_formula = transformer_display.transformToFragment(result, output.ownerDocument);
      if (debug)
        {
          var result_as_string;
          result_as_string = serializer.serializeToString(result);
          output.appendChild(document.createTextNode(result_as_string));
          output.appendChild(e(xhtml_namespace, "br"));
          output.appendChild(document
                             .createTextNode(serializer
                                             .serializeToString(displayed_formula)
                                             )
                             );
          output.appendChild(e(xhtml_namespace, "br"));
          result_as_string = parser.openmath_to_linear(result);
          if (result_as_string)
            result_as_string = serializer.serializeToString(result_as_string);
          output.appendChild(document.createTextNode(result_as_string));
          output.appendChild(e(xhtml_namespace, "br"));
        }
      
      if (displayed_formula.firstChild)
        {
          displayed_formula = displayed_formula.firstChild;
        }
      while (displayed_formula)
        {
          displayed_formula.parentNode.removeChild(displayed_formula);
          output.appendChild(displayed_formula);
          displayed_formula = displayed_formula.nextSibling;
        }
      
      results[result_string] = step;
      if (kernel) result = kernel.evaluate_step(result, document, "simplify/arithmetic");
      result_string = serializer.serializeToString(result);
      
      ++step;
    } while (! (result_string in results)/* && confirm("Next step?")*/);
  
  var result_child;
  result_child = result.firstChild;
  do
    {
      if (result_child
          && "localName" in result_child
          && "OMOBJ" == result_child.localName)
        {
          var command_list;
          command_list = new Array();
          var command_iterator;
          command_iterator =
            xpath("descendant::om:OMS[@cd='org.matracas.algebra']",
                  result_child);
          if (command_iterator)
            {
              var command;
              while ((command = command_iterator.iterateNext()))
                command_list.push(command);
              var expressions_to_plot;
              expressions_to_plot = new Array();
              for (i in command_list)
                {
                  command = command_list[i];
                  switch (command.getAttribute("name"))
                    {
                    case "plot":
                      output.appendChild(e(xhtml_namespace, "br"));
                      expressions_to_plot.push(command.parentNode);
                      break;
                    case "debug":
                      if (command.nextSibling)
                        {
                          switch (command.nextSibling.getAttribute("name"))
                            {
                            case "true":  debug = true;  break;
                            case "false": debug = false; break;
                              // TODO: default, syntax error.
                            }
                        }
                      // TODO: else, syntax error.
                      break;
                    }
                }
              if (expressions_to_plot.length > 0)
                {
                  if (1 == expressions_to_plot.length)
                    {
                      expressions_to_plot = expressions_to_plot[0];
                    }
                  var max_auto_width;
                  var max_auto_height;
                  max_auto_width  = window.parent.innerWidth / 2.1;
                  max_auto_height = window.parent.innerHeight / 1.25;
                  var plotter;
                  plotter = new Plotter(kernel, expressions_to_plot,
                                        max_auto_width, max_auto_height);
                  output.appendChild(plotter.get_panel());
                }
            }
        }
    } while ((result_child = result_child.nextSibling));
}

function enter_input(input)
{
  current_input = input;
  var current_index;
  current_index = parseInt(input.id.replace("input", ""));
  
  window.parent.set_title(input.value);
  update_notebook_entry(current_index, input);
  
  ++current_index;
  current_input = get_input_text_field(current_index);
  if (!current_input)
    {
      append_notebook_entry(current_index);
      current_input = get_input_text_field(current_index);
    }
  current_input.focus();
}

function insert(text)
{
  var template;
  template = palette_templates[text];// Specific for each CAS syntax.
  if (!template) template = text;
  
  var input;
  input = current_input;// This can change if it loses focus.
  if (input)
    {
      var start;
      var end;
      start = input.selectionStart;
      end   = input.selectionEnd;
      var placeholder_index;
      placeholder_index = template.indexOf("\uEEEE");
      var cursor;
      if (placeholder_index >= 0)
        {
          cursor = start + placeholder_index;
          text = template.replace("\uEEEE", input.value.substring(start, end));
          if (end > start)
            {
              // We replace the placeholder with the selection, so we move to
              // the end of the inserted text to continue writing.
              cursor = start + text.length;
            }
          else
            {
              // There is no selection, so the user would most likely want to
              // start filling in the missing content.
              cursor = start + placeholder_index;
            }
        }
      else
        {
          text = template;
          cursor = start + text.length;
        }
      input.value = input.value.substring(0, start) + text + input.value.substring(end);
      resize_input(input);
      input.setSelectionRange(cursor, cursor);
      input.focus();
    }
  else
    {
      message("No input");
    }
}

function status_message(message)
{
  if ("status_message" in window.parent) window.parent.status_message(message);
}

function message(message)
{
  if ("message" in window.parent) window.parent.message(message);
  else                            alert(message);
}
