///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2006 Alberto González Palomo
//   Author: Alberto González Palomo <http://www.matracas.org>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////

const xhtml_namespace = "http://www.w3.org/1999/xhtml";

const xml_serializer = new XMLSerializer();

var parser;

function init()
{
  parser = new Parser();
  initialize_parser(parser);
  
  var current_input;
  current_input = append_new_input(1);
  current_input.focus();
}

function keydown_input(event)
{
  var input;
  input = event.target;
  
  var at_end_of_input;
  at_end_of_input = (input.selectionStart == input.value.length);
  
  switch (event.keyCode)
    {
    case event.DOM_VK_BACK_SPACE:
      input.setAttribute("rows", input.value.replace(/./g, "").length + 1);
      break;
    case event.DOM_VK_RETURN:
      input.setAttribute("rows", input.value.replace(/./g, "").length + 1);
      break;
    }
}

function keyup_input(event)
{
  var input;
  input = event.target;
  
  var at_end_of_input;
  at_end_of_input = (input.selectionStart == input.value.length);
  
  switch (event.keyCode)
    {
    case event.DOM_VK_RETURN:
      if (input.value && at_end_of_input && input.value.match(/;\s*$/))
        {
          enter_input(input);
        }
      break;
    case event.DOM_VK_ENTER:
      enter_input(input);
      break;
    case event.DOM_VK_BACK_SPACE:
      //input.setAttribute("rows", input.value.replace(/./g, "").length + 1);
      break;
    }
}

function make_row(type, content)
{
  var row;
  row = document.createElementNS(xhtml_namespace, "tr");
  row.setAttribute("class", type);
  var prompt;
  prompt = document.createElementNS(xhtml_namespace, "td");
  prompt.setAttribute("class", type + "-prompt");
  var container;
  container = document.createElementNS(xhtml_namespace, "td"); 
  container.setAttribute("class", type + "-container");
  container.appendChild(content);
  
  row.appendChild(prompt);
  row.appendChild(container);
  
  return row;
}

function enter_input(input)
{
  var current_input;
  current_input = input;
  var output;
  var output_id;
  output_id = input.id.replace('input', 'output');
  output = document.getElementById(output_id);
  if (!output)
    {
      output = document.createElementNS(xhtml_namespace, 'div');
      output.setAttribute("class", "output");
      output.setAttribute('id', output_id);
      input.parentNode.parentNode.parentNode
        .appendChild(make_row("output", output));
    }
  
  var result;
  result = evaluate(input);
  if (/\)error /.test(result))
    {
      output.innerHTML = result.substring(6);
    }
  else
    {
      output.innerHTML = result;
      var next_input_index;
      next_input_index = parseInt(input.id.replace("input", "")) + 1;
      current_input = document.getElementById("input" + next_input_index);
      if (!current_input) current_input = append_new_input(next_input_index);
    }
  
  current_input.focus();
}

function append_new_input(index)
{
  var new_input;
  //new_input = new XML('<input class="input" onchange="changed_input(this)"/>');
  new_input = document.createElementNS(xhtml_namespace, "textarea");
  new_input.setAttribute("rows", "0");
  new_input.setAttribute("onkeydown", "keydown_input(event)");
  new_input.setAttribute("onkeyup",     "keyup_input(event)");
  new_input.setAttribute("class", "input");
  new_input.setAttribute("id", "input" + index);
  
  var new_computation;
  new_computation = document.createElementNS(xhtml_namespace, "table");
  new_computation.setAttribute("class", "computation");
  new_computation.setAttribute("width", "100%");
  var computation_container;
  computation_container = document.createElementNS(xhtml_namespace, "div");
  computation_container.setAttribute("class", "computation-container");
  
  new_computation.appendChild(make_row("input", new_input));
  computation_container.appendChild(new_computation);
  document.getElementById("notebook-body").appendChild(computation_container);
  
  return new_input;
}

function evalaute(input)
{
  var result;
  result = parser.parse_to_XML(text);
  return xml_serializer.serializeToString(result);
}

function evaluate_arithmetic(input)
{
  var input_expression;
  
  if (input.value.match(/%/))
    {
      var previous_input;
      previous_input = document
        .getElementById("input" + (parseInt(input.id.replace("input", ""))-1));
      if (previous_input)
        {
          if (previous_input == input)
            {
              alert("Error: infinite recursion found.");
              return "[error]";
            }
          
          input_expression = input.value.replace(/%/g,
                                                 evaluate(previous_input)
                                                 );
        }
      else
        {
          input_expression = "";
        }
    }
  else
    {
      input_expression = input.value;
    }
  
  var result;
  try
    {
      result = eval(input_expression);
    }
  catch (exception)
    {
      result = ")error " + exception + ", in <code class='user-input-quote'>" + input_expression + "</code>";
    }
  
  return result;
}
