///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2006 Alberto González Palomo
//   Author: Alberto González Palomo <http://www.matracas.org>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////

const xul_namespace = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
const xhtml_namespace = 'http://www.w3.org/1999/xhtml';

register_namespace_map({"xul":xul_namespace});

function make_row(type, content, index)
{
  var row;
  row = document.createElementNS(xhtml_namespace, "tr");
  row.setAttribute("class", type);
  var prompt;
  prompt = document.createElementNS(xhtml_namespace, "td");
  prompt.setAttribute("valign", "top");
  prompt.setAttribute("class", type + "-prompt");
  var prompt_label;
  switch (type)
    {
    case "input":
      prompt_label = "(C" + index + ")";
      break;
    case "output":
      prompt_label = "(D" + index + ")";
      break;
    }
  if (prompt_label)
    {
      prompt.appendChild(document.createTextNode(prompt_label));
    }
  var container;
  container = document.createElementNS(xhtml_namespace, "td"); 
  container.setAttribute("class", type + "-container");
  container.appendChild(content);
  var closure;
  closure = document.createElementNS(xhtml_namespace, "td");
  closure.setAttribute("class", type + "-closure");
  
  row.appendChild(prompt);
  row.appendChild(container);
  row.appendChild(closure);
  
  return row;
}

function new_notebook_entry(index)
{
  var input_text_field;
  //input_text_field = new XML('<input class="input" onchange="changed_input(this)"/>');
  input_text_field = document.createElementNS(xhtml_namespace, "textarea");
  input_text_field.setAttribute("rows", "1");
  input_text_field.setAttribute("style", "width:100%");
  input_text_field.setAttribute("onkeydown", "keydown_input(event)");
  input_text_field.setAttribute("onkeyup",     "keyup_input(event)");
  input_text_field.setAttribute("class", "input-text-field");
  input_text_field.setAttribute("id", "input" + index);
  
  var new_computation;
  new_computation = document.createElementNS(xhtml_namespace, "table");
  new_computation.setAttribute("width", "100%");
  new_computation.setAttribute("class", "computation-frame");
  var notebook_entry;
  notebook_entry = document.createElementNS(xhtml_namespace, "table");
  notebook_entry.setAttribute("width", "100%");
  notebook_entry.setAttribute("class", "notebook-entry");
  
  new_computation.appendChild(make_row("input", input_text_field, index));
  notebook_entry.appendChild(make_row("computation", new_computation, index));
  
  return notebook_entry;
}

function create_output_node_for_notebook_entry(input, index)
{
  var new_output;
  new_output = document.createElementNS(xhtml_namespace, 'div');
  new_output.setAttribute("class", "output-content");
  new_output.setAttribute('id', "output" + index);
  
  var node;
  node = input;
  while (node.parentNode && node.getAttribute("class") != "input")
    node = node.parentNode;
  if (node.getAttribute("class") == "input")
    {
      node = node.parentNode;
      node.appendChild(make_row("output", new_output, index));
    }
  else
    {
      alert("input container not found: " + node.nodeName);
    }
  
  return new_output;
}

function keydown_input(event)
{
  var input;
  input = event.target;
  
  var at_end_of_input;
  at_end_of_input = (input.selectionStart == input.value.length);
  var lines;
  lines = input.value.replace(/./g, "").length + 1;
  
  switch (event.keyCode)
    {
    case event.DOM_VK_BACK_SPACE:
      resize_input(input);
      break;
    case event.DOM_VK_RETURN:
      if (input.value
          && ((at_end_of_input && input.value.match(/;\s*$/))
              || (event.shiftKey /* Mma */)
              )
          )
        {
          resize_input(input);
          event.preventDefault();
          event.stopPropagation();
        }
      else
        {
          resize_input(input);
        }
      break;
    }
}

function keyup_input(event)
{
  var input;
  input = event.target;
  
  var at_end_of_input;
  at_end_of_input = (input.selectionStart == input.value.length);
  
  switch (event.keyCode)
    {
    case event.DOM_VK_RETURN:
      if (input.value
          && ((at_end_of_input && input.value.match(/;\s*$/))
              || (event.shiftKey /* Mma */)
              )
          )
        {
          input.value = input.value.replace(/\s$/g, "");
          resize_input(input);
          enter_input(input);
        }
      break;
    case event.DOM_VK_ENTER:
      enter_input(input);
      break;
    case event.DOM_VK_BACK_SPACE:
      resize_input(input);
      break;
    }
}

var palette_templates =
  {"square_root":"\u221A(\uEEEE)",
   "x_squared":"(\uEEEE)^2",
   "x_cubed":"(\uEEEE)^3",
   "inverse_of_x":"1/(\uEEEE)",
   "fraction":"(\uEEEE)/",
   "sinus":"sin(\uEEEE)",
   "cosinus":"cos(\uEEEE)",
   "tangent":"tan(\uEEEE)",
   "plot_sinus":"plot(sin(x), [x, -2*%pi, 2*%pi]);",
   "plot_cosinus":"plot(cos(x), [x, -2*%pi, 2*%pi]);",
   "plot_tangent":"plot(tan(x), [x, -2*%pi, 2*%pi]);",
   "plot_inverse_of_x":"plot(1/x, [x, -2, 2]);",
   "plot_harmonics_1_3":"plot(sin(x)+sin(x*3)/2, [x, -2*%pi, 2*%pi]);",
   "plot_harmonics_1_2":"plot(sin(x)+sin(x*2)/2, [x, -2*%pi, 2*%pi]);"
  };
