///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2006 Alberto González Palomo
//   Author: Alberto González Palomo <http://www.matracas.org>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////

interface_style = "mma";

function make_row(type, content, index)
{
  var prompt_label;
  var box_tag;
  var container;
  
  switch (type)
    {
    case "input":
      prompt_label = "In[" + index + "]:=";
      container = content;
      container.setAttribute("class", type + "-container");
      container.setAttribute("flex", "1");
      break;
    case "output":
      prompt_label = "Out[" + index + "]=";
      container = document.createElementNS(xul_namespace, "vbox");
      container.setAttribute("class", type + "-container");
      container.setAttribute("flex", "1");
      container.appendChild(content);
      break;
    case "computation":
      prompt_label = null;
      container = content;
      container.setAttribute("class", type + "-container");
      break;
    default:
      alert("make_row(): Unexpected type: " + type);
      break;
    }
  
  var row;
  row = document.createElementNS(xul_namespace, "hbox");
  row.setAttribute("class", type);
  var prompt;
  prompt = document.createElementNS(xul_namespace, "vbox");
  prompt.setAttribute("class", type + "-prompt");
  if (prompt_label)
    {
      var label;
      label = document.createElementNS(xul_namespace, "label");
      label.appendChild(document.createTextNode(prompt_label));
      prompt.appendChild(label);
    }
  var closure;
  closure = document.createElementNS(xul_namespace, "vbox");
  closure.setAttribute("class", type + "-closure");
  
  row.appendChild(prompt);
  row.appendChild(container);
  row.appendChild(closure);
  
  return row;
}

function new_notebook_entry(index)
{
  var input_text_field;
  //input_text_field = new XML('<input class="input" onchange="changed_input(this)"/>');
  input_text_field = document.createElementNS(xul_namespace, "textbox");
  input_text_field.setAttribute("multiline", "true");
  input_text_field.setAttribute("rows", "1");
  input_text_field.setAttribute("onkeydown", "keydown_input(event)");
  input_text_field.setAttribute("onkeyup",     "keyup_input(event)");
  input_text_field.setAttribute("onfocus",      "focus_input(this)");
  //input_text_field.setAttribute("onblur",        "blur_input(this)");
  input_text_field.setAttribute("class", "input-text-field");
  input_text_field.setAttribute("id", "input" + index);
  
  var new_computation;
  new_computation = document.createElementNS(xul_namespace, "vbox");
  new_computation.setAttribute("flex", "1");
  new_computation.setAttribute("class", "computation-frame");
  var notebook_entry;
  //notebook_entry = document.createElementNS(xul_namespace, "hbox");
  //notebook_entry.setAttribute("flex", "1");
  //notebook_entry.setAttribute("class", "notebook-entry");
  
  new_computation.appendChild(make_row("input", input_text_field, index));
  notebook_entry = make_row("computation", new_computation);
  
  return notebook_entry;
}

function create_output_node_for_notebook_entry(input, index)
{
  var new_output;
  new_output = document.createElementNS(xhtml_namespace, "p");
  new_output.setAttribute("class", "output-content");
  new_output.setAttribute('id', "output" + index);
  
  var node;
  node = input;
  while (node.parentNode && node.getAttribute("class") != "input")
    node = node.parentNode;
  if (node.getAttribute("class") == "input")
    {
      node = node.parentNode;
      node.appendChild(make_row("output", new_output, index));
    }
  else
    {
      alert("input container not found: " + node.nodeName);
    }
  
  return new_output;
}

function keydown_input(event)
{
  var input;
  input = event.target;
  
  var at_end_of_input;
  at_end_of_input = (input.selectionStart == input.value.length);
  var lines;
  lines = input.value.replace(/./g, "").length + 1;
  
  switch (event.keyCode)
    {
    case event.DOM_VK_BACK_SPACE:
      resize_input(input);
      break;
    case event.DOM_VK_RETURN:
      if (input.value
          && ((at_end_of_input && input.value.match(/;\s*$/))
              || (event.shiftKey /* Mma */)
              )
          )
        {
          resize_input(input);
          event.preventDefault();
          event.stopPropagation();
        }
      else
        {
          resize_input(input);
        }
      break;
    }
}

function keyup_input(event)
{
  var input;
  input = event.target;
  
  var at_end_of_input;
  at_end_of_input = (input.selectionStart == input.value.length);
  
  switch (event.keyCode)
    {
    case event.DOM_VK_RETURN:
      if (input.value
          && ((at_end_of_input && input.value.match(/;\s*$/))
              || (event.shiftKey /* Mma */)
              )
          )
        {
          input.value = input.value.replace(/\s$/g, "");
          resize_input(input);
          enter_input(input);
          event.preventDefault();
          event.stopPropagation();
        }
      break;
    case event.DOM_VK_ENTER:
      enter_input(input);
      break;
    case event.DOM_VK_BACK_SPACE:
      resize_input(input);
      break;
    }
}

var palette_templates =
  {"square_root":"\u221A(\uEEEE)",
   "x_squared":"(\uEEEE)^2",
   "x_cubed":"(\uEEEE)^3",
   "inverse_of_x":"1/(\uEEEE)",
   "fraction":"(\uEEEE)/",
   "sinus":"Sin[\uEEEE]",
   "cosinus":"Cos[\uEEEE]",
   "tangent":"Tan[\uEEEE]",
   "plot_sinus":"Plot[Sin[x], {x, -2*Pi, 2*Pi}]",
   "plot_cosinus":"Plot[Cos[x], {x, -2*Pi, 2*Pi}]",
   "plot_tangent":"Plot[Tan[x], {x, -2*Pi, 2*Pi}]",
   "plot_inverse_of_x":"Plot[1/x, {x, -6, 6}]",
   "plot_hemicircle":"Plot[√[1-x^2], {x, -1, 1}]",
   "plot_circle":"Plot[{√[100-x^2], -√[100-x^2]}, {x, -10, 10}]",
   "plot_harmonics_1_3":"Plot[Sin[x]+Sin[x*3]/2, {x, -2*Pi, 2*Pi}]",
   "plot_harmonics_1_2":"Plot[Sin[x]+Sin[x*2]/2, {x, -2*Pi, 2*Pi}]",
   "plot_multi_circle":"{Plot[{√[100-x^2], -√[100-x^2]}, {x, -10, 10}],\n Plot[{√[9-x^2], -√[9-x^2]}, {x, -3, 3}]}",
   "plot_multi_trig":"Plot[{Tan[x]/4, Sin[x]+Sin[x*2]/2, Sin[x], Cos[x]}, {x, -2*Pi, 2*Pi}]"
  };
